# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Definition of executor."""

import datetime
from threading import Lock, Thread
from typing import Any, List, Optional, Union

from neural_compressor.ux.utils.logger import log
from neural_compressor.ux.utils.proc import Proc
from neural_compressor.ux.utils.processes import NCProcesses
from neural_compressor.ux.web.communication import MessageQueue

LOCK = Lock()


class Executor:
    """Executor class provide execute shell command."""

    def __init__(
        self,
        workspace_path: str,
        subject: str,
        data: Optional[dict] = None,
        send_response: bool = True,
        log_name: Optional[str] = None,
        additional_log_names: List[str] = [],
    ) -> None:
        """
        Bash Executor constructor.

        :param args: arguments for parser execution
        """
        if data:
            self._request_id = str(data.get("id", ""))
        self._log_name = log_name
        self._additional_log_names = additional_log_names
        self._workdir = workspace_path
        self._mq = MessageQueue()
        self._subject = subject
        self._send_response = send_response
        self.data = data
        self.time_start: Optional[datetime.datetime] = None
        self.time_stop: Optional[datetime.datetime] = None

    def call_one(
        self,
        args: List[Any],
        logger: Optional[Any] = None,
        executable: Optional[Any] = None,
        shell: bool = False,
        cwd: Optional[str] = None,
        env: Optional[dict] = None,
        universal_newlines: bool = False,
        startupinfo: Optional[Any] = None,
        creationflags: int = 0,
        processes: Optional[Any] = None,
        ignore_exit_codes: Union[list, Any] = None,
        pid: Optional[str] = None,
    ) -> None:
        """
        Execute single call for process.

        :param args:
        :param logger:
        :param executable:
        :param shell:
        :param cwd:
        :param env:
        :param universal_newlines:
        :param startupinfo:
        :param creationflags:
        :param processes:
        :param ignore_exit_codes:
        :param pid:
        """
        proc = None

        try:
            log.debug("Exec %s ", " ".join(map(str, args)))
            proc = Proc(
                self.workdir,
                pid=pid,
                request_id=self.request_id,
                filename=self.log_name,
                additional_log_names=self.additional_log_names,
            )
            if self._send_response:
                self._mq.post_success(
                    "_".join([self._subject, "start"]),
                    self.data,
                )
            proc.run(
                args,
                executable,
                shell,
                cwd,
                env,
                universal_newlines,
                startupinfo,
                creationflags,
                ignore_exit_codes,
            )
            self.time_start = proc.time_start
            self.time_stop = proc.time_stop

        except Exception:
            if self._send_response:
                self._mq.post_error(
                    self._subject,
                    {"message": Exception, "id": self.request_id},
                )
            log.exception("Unexpected error for command line %s", args)
        try:
            LOCK.acquire()
            if processes is None:
                processes = []
            processes.append(proc)
        finally:
            LOCK.release()
            log.debug("DONE")

    def call(
        self,
        args: List[Any],  # type: ignore
        logger: Optional[Any] = None,
        executable: Optional[Any] = None,
        shell: bool = False,
        cwd: Optional[str] = None,
        env: Optional[dict] = None,
        universal_newlines: bool = False,
        startupinfo: Optional[Any] = None,
        creationflags: int = 0,
        env_args: Optional[list] = None,
        ignore_exit_codes: Union[list, Any] = None,
        pid: Optional[str] = None,
    ) -> NCProcesses:
        """
        Execute multiple calls for process.

        :param args:
        :param logger:
        :param executable:
        :param shell:
        :param cwd:
        :param env:
        :param universal_newlines:
        :param startupinfo:
        :param creationflags:
        :param ignore_exit_codes:
        :param pid:
        """
        # update output directory to current stage
        self.refresh_workdir()

        threads = []
        processes = NCProcesses()

        if not self.is_multi_commands(args):
            args = [args]

        for arg in args:
            threads.append(
                Thread(
                    target=self.call_one,
                    args=(
                        arg,
                        logger,
                        executable,
                        shell,
                        cwd,
                        env_args,
                        universal_newlines,
                        startupinfo,
                        creationflags,
                        processes,
                        ignore_exit_codes,
                        pid,
                    ),
                    daemon=True,
                ),
            )

        # Start all threads
        for command_thread in threads:
            command_thread.start()

        # Wait for all of them to finish
        for command_thread in threads:
            command_thread.join()

        return processes

    @property
    def process_duration(self) -> Optional[float]:
        """Return duration of the process in [s]."""
        if self.time_start and self.time_stop:
            duration = self.time_stop - self.time_start
            return duration.total_seconds()
        return None

    @property
    def workdir(self) -> str:
        """
        Property concat workdir path.

        :return: workdir path
        """
        return self._workdir

    @property
    def request_id(self) -> Optional[str]:
        """
        Property contain info about request.

        :return: request id
        """
        return self._request_id

    @property
    def log_name(self) -> Optional[str]:
        """
        Property contain info about output file name.

        :return: requested log filename
        """
        return self._log_name

    @property
    def additional_log_names(self) -> List[str]:
        """
        Property contain info about additional output file names.

        :return: list of additional log filenames
        """
        return self._additional_log_names

    def refresh_workdir(self) -> str:
        """
        Property returns workdir method.

        :return: workdir path
        """
        return self.workdir

    @staticmethod
    def is_multi_commands(args: list) -> bool:
        """
        Check type of execution.

        :param args: arguments for parser execution
        :return: bool value True if args are in list else False
        """
        for arg in args:
            if not isinstance(arg, list):
                return False
        # all elements must be lists
        return True



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Configuration to yaml."""

from pathlib import Path
from typing import Any, Dict, List, Union

from neural_compressor.ux.components.model.repository import ModelRepository
from neural_compressor.ux.utils.exceptions import ClientErrorException
from neural_compressor.ux.utils.hw_info import HWInfo
from neural_compressor.ux.utils.workload.config import Config


def get_predefined_configuration(
    data: Dict[str, Any],
) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
    """Get configuration."""
    from neural_compressor.ux.utils.utils import (
        get_framework_from_path,
        get_predefined_config_path,
    )

    model_path = data.get("model_path", "")
    if not ModelRepository.is_model_path(model_path):
        raise ClientErrorException(
            f"Could not find model in specified path: {model_path}.",
        )

    model_name = Path(model_path).stem

    domain = data.get("domain", None)
    domain_flavour = data.get("domain_flavour", "")

    if not domain:
        raise ClientErrorException("Domain is not defined!")

    framework = get_framework_from_path(model_path)
    if framework is None:
        raise ClientErrorException(
            f"Could not find framework for specified model {model_name} in path {model_path}.",
        )

    config = Config()
    predefined_config_path = get_predefined_config_path(framework, domain, domain_flavour)
    config.load(predefined_config_path)

    update_config_to_machine_specification(config)

    return {
        "config": config.serialize(),
        "framework": framework,
        "name": model_name,
        "domain": domain,
    }


def update_config_to_machine_specification(config: Config) -> None:
    """Change Config values based on local machine."""
    hwinfo = HWInfo()

    cores_per_socket = hwinfo.cores_per_socket

    if not config.evaluation:
        return

    if config.evaluation.accuracy and config.evaluation.accuracy.dataloader:
        config.evaluation.accuracy.dataloader.batch_size = cores_per_socket

    if config.evaluation.performance:
        config.evaluation.performance.configs.cores_per_instance = 4
        config.evaluation.performance.configs.num_of_instance = cores_per_socket // 4

        if config.evaluation.performance.dataloader:
            config.evaluation.performance.dataloader.batch_size = 1



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Graph classes."""

import unittest
import uuid

from neural_compressor.ux.components.graph.graph import Graph
from neural_compressor.ux.components.graph.node import Node


def _get_random_string() -> str:
    """Create random string to be used."""
    return uuid.uuid4().hex


def _get_random_node() -> Node:
    """Create a Node with random values."""
    return Node(id=_get_random_string(), label=_get_random_string())


class TestGraph(unittest.TestCase):
    """Test Graph class."""

    def test_empty_graph_has_no_nodes(self) -> None:
        """Test if empty graph has no nodes."""
        graph = Graph()
        self.assertEqual([], graph.nodes)

    def test_empty_graph_has_no_edges(self) -> None:
        """Test if empty graph has no edges."""
        graph = Graph()
        self.assertEqual([], graph.edges)

    def test_graph_with_one_node_returns_only_one_node(self) -> None:
        """Test if empty graph with one node returns it correctly."""
        node = _get_random_node()
        graph = Graph()
        graph.add_node(node)

        self.assertEqual([node], graph.nodes)

    def test_adding_edge_when_no_nodes_does_not_add_an_adge(self) -> None:
        """Test if adding an edge to empty graph fails as expected."""
        source_id = _get_random_string()
        target_id = _get_random_string()

        graph = Graph()
        result = graph.add_edge(source_id=source_id, target_id=target_id)

        self.assertEqual(False, result)
        self.assertEqual([], graph.edges)

    def test_adding_edge_between_nodes_works(self) -> None:
        """Test adding an edge."""
        source = _get_random_node()
        target = _get_random_node()

        graph = Graph()
        graph.add_node(source)
        graph.add_node(target)
        result = graph.add_edge(source_id=source.id, target_id=target.id)

        self.assertEqual(True, result)

        edges = graph.edges
        self.assertEqual(1, len(edges))

        edge = edges[0]
        self.assertEqual(source, edge._source)
        self.assertEqual(target, edge._target)

    def test_comparing_to_something_else_fails(self) -> None:
        """Test if comparing Graph to something other fails."""
        with self.assertRaises(NotImplementedError):
            Graph() == 1


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .op import Operator, operator_registry
from .tensor import Tensor


@operator_registry(operator_type='ModelDataset')
class ModelDataset(Operator):
    def __init__(self):
        super().__init__()

    def set_attr(self, framework, node):
        if framework == 'tensorflow':
            from ..tf_utils import TF_DTYPE_ID
            output_shapes = []
            shape_list = node.attr['output_shapes'].list.shape
            for each in shape_list:
                tmp = []
                shape = each.dim
                for s in shape:
                    tmp.append(s.size)
                output_shapes.append(tmp)
            self._attr['output_shapes'] = output_shapes
            output_types = node.attr['output_types'].list.type
            output_types = [TF_DTYPE_ID[i] for i in output_types]
            self._attr['output_types'] = output_types



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .op import Operator, operator_registry
from .tensor import Tensor
from ..graph_utils import list2str


# tf.transpose(a, perm=None, conjugate=False, name='transpose')
# The returned tensor's dimension i will correspond to the input dimension perm[i].
# If perm is not given, it is set to (n-1...0), where n is the rank of the input tensor.
# Hence by default, this operation performs a regular matrix transpose on 2-D input Tensors.
# If conjugate is True and a.dtype is either complex64 or complex128 then the values of a are
# conjugated and transposed.
# normally, we don't need the param 'conjugate'
@operator_registry(operator_type='Transpose')
class Transpose(Operator):
    def __init__(self):
        super().__init__()

    def set_attr(self, framework, node):

        if framework == "tensorflow":
            if self._input_tensors[1].source_op == []:
                dst_perm = list(self._input_tensors[1].data)
                self._attr['dst_perm'] = list2str(dst_perm)
                src_perm = [i for i in range(len(dst_perm))]
                self._attr['src_perm'] = list2str(src_perm)
                _ = self._input_tensors.pop()

        if framework == "onnxruntime":
            dst_perm = node.attribute[0].ints
            src_perm = [i for i in range(len(dst_perm))]
            self._attr['src_perm'] = list2str(src_perm)
            self._attr['dst_perm'] = list2str(dst_perm)



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Workdir class."""

import glob
import json
import logging
import os
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from neural_compressor.ux.utils.templates.metric import Metric
from neural_compressor.ux.utils.workload.workloads_list import WorkloadInfo
from neural_compressor.ux.web.configuration import Configuration


class Workdir:
    """Metric represents data which is sent in get_workloads_list endpoint."""

    def __init__(
        self,
        request_id: Optional[str] = None,
        project_name: Optional[str] = None,
        model_path: Optional[str] = None,
        input_precision: Optional[str] = None,
        model_output_path: Optional[str] = None,
        output_precision: Optional[str] = None,
        mode: Optional[str] = None,
        metric: Optional[Union[dict, Metric]] = None,
        overwrite: bool = True,
        created_at: Optional[str] = None,
    ) -> None:
        """Initialize workdir class."""
        configuration = Configuration()
        workspace_path = configuration.workdir
        self.workdir_path = os.path.join(os.environ.get("HOME", ""), ".neural_compressor")
        self.ensure_working_path_exists()
        self.workloads_json = os.path.join(self.workdir_path, "workloads_list.json")
        self.request_id = request_id
        self.workload_path: str
        if not metric:
            metric = Metric()

        if os.path.isfile(self.workloads_json):
            self.workloads_data = self.load()
        else:
            self.workloads_data = {
                "active_workspace_path": workspace_path,
                "workloads": {},
                "version": "3",
            }

        workload_data = self.get_workload_data(request_id)
        if workload_data:
            self.workload_path = workload_data.get("workload_path", "")
        elif workspace_path and request_id:
            workload_name = request_id
            if model_path:
                workload_name = "_".join(
                    [
                        Path(model_path).stem,
                        request_id,
                    ],
                )
            self.workload_path = os.path.join(
                workspace_path,
                "workloads",
                workload_name,
            )

        if request_id and overwrite:
            self.update_data(
                request_id=request_id,
                project_name=project_name,
                model_path=model_path,
                input_precision=input_precision,
                model_output_path=model_output_path,
                output_precision=output_precision,
                mode=mode,
                metric=metric,
                created_at=created_at,
            )

    def load(self) -> dict:
        """Load json with workloads list."""
        logging.info(f"Loading workloads list from {self.workloads_json}")
        with open(self.workloads_json, encoding="utf-8") as workloads_list:
            self.workloads_data = json.load(workloads_list)
            return self.workloads_data

    def dump(self) -> None:
        """Dump workloads information to json."""
        with open(self.workloads_json, "w") as f:
            json.dump(self.workloads_data, f, indent=4)

        logging.info(f"Successfully saved workload to {self.workloads_json}")

    def ensure_working_path_exists(self) -> None:
        """Set workdir if doesn't exist create."""
        os.makedirs(self.workdir_path, exist_ok=True)

    def map_to_response(self) -> dict:
        """Map to response data."""
        data: Dict[str, Any] = {
            "workspace_path": self.get_active_workspace(),
            "workloads_list": [],
        }
        for key, value in self.workloads_data["workloads"].items():
            data["workloads_list"].append(value)
        return data

    def update_data(
        self,
        request_id: str,
        project_name: Optional[str] = None,
        model_path: Optional[str] = None,
        input_precision: Optional[str] = None,
        model_output_path: Optional[str] = None,
        output_precision: Optional[str] = None,
        mode: Optional[str] = None,
        metric: Optional[Union[Dict[str, Any], Metric]] = Metric(),
        status: Optional[str] = None,
        execution_details: Optional[Dict[str, Any]] = None,
        created_at: Optional[str] = None,
    ) -> None:
        """Update data in workloads.list_json."""
        self.load()
        existing_data = self.get_workload_data(request_id)
        if project_name is None:
            project_name = existing_data.get("project_name", None)
        if created_at is None:
            created_at = existing_data.get("created_at", None)
        workload_info = WorkloadInfo(
            workload_path=self.workload_path,
            request_id=request_id,
            project_name=project_name,
            model_path=model_path,
            input_precision=input_precision,
            model_output_path=model_output_path,
            output_precision=output_precision,
            mode=mode,
            metric=metric,
            status=status,
            code_template_path=self.template_path,
            execution_details=execution_details,
            created_at=created_at,
        ).serialize()
        self.workloads_data["workloads"][request_id] = workload_info
        self.dump()

    def set_workload_status(self, request_id: str, status: str) -> None:
        """Set status of a given Workload."""
        self.load()
        existing_data = self.get_workload_data(request_id)
        if not existing_data:
            return
        self.workloads_data["workloads"][request_id]["status"] = status
        self.dump()

    def update_metrics(
        self,
        request_id: Optional[str],
        metric_data: Dict[str, Any],
    ) -> None:
        """Update metric data in workload."""
        self.load()
        self.workloads_data["workloads"][request_id].get("metric", {}).update(
            metric_data,
        )
        self.workloads_data["workloads"][request_id].update(metric_data)
        self.dump()

    def update_execution_details(
        self,
        request_id: Optional[str],
        execution_details: Dict[str, Any],
    ) -> None:
        """Update metric data in workload."""
        self.load()
        self.workloads_data["workloads"][request_id].get(
            "execution_details",
            {},
        ).update(execution_details)
        self.dump()

    def get_active_workspace(self) -> str:
        """Get active workspace."""
        configuration = Configuration()
        path = self.workloads_data.get("active_workspace_path", configuration.workdir)
        return path

    def set_active_workspace(self, workspace_path: str) -> None:
        """Set active workspace."""
        self.workloads_data["active_workspace_path"] = workspace_path
        self.dump()

    def set_code_template_path(self, code_template_path: str) -> None:
        """Set code_template_path."""
        self.workloads_data["workloads"][self.request_id][
            "code_template_path"
        ] = code_template_path
        self.dump()

    def clean_logs(self) -> None:
        """Clean log files."""
        log_files = [os.path.join(self.workload_path, "output.txt")]
        log_files.extend(
            glob.glob(
                os.path.join(self.workload_path, "*.proc"),
            ),
        )
        log_files.extend(
            glob.glob(
                os.path.join(self.workload_path, "*performance_benchmark.txt"),
            ),
        )
        for file in log_files:
            if os.path.exists(file):
                os.remove(file)

    def clean_status(
        self,
        status_to_clean: Optional[str] = None,
        requests_id: Optional[List[str]] = [],
    ) -> None:
        """Clean status for workloads according to passed parameters."""
        for workload_id, workload_params in self.workloads_data["workloads"].items():
            if requests_id and workload_id not in requests_id:
                continue
            self._clean_workload_status(
                workload_params,
                status_to_clean,
            )
        self.dump()

    @staticmethod
    def _clean_workload_status(workload: dict, status: Optional[str]) -> None:
        """Clean specified workload status."""
        workload_status = workload.get("status")
        if status and workload_status != status:
            return
        workload["status"] = ""

    @property
    def template_path(self) -> Optional[str]:
        """Get template_path."""
        return self.get_workload_data(self.request_id).get("code_template_path", None)

    def get_workload_data(self, request_id: Optional[str]) -> dict:
        """Return data of given Workload."""
        return self.workloads_data.get("workloads", {}).get(request_id, {})



#
#  -*- coding: utf-8 -*-
#
#  Copyright (c) 2021 Intel Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

import numpy as np

def collate_preds(results):
    batch = results[0]
    if isinstance(batch, list):
        results = zip(*results)
        collate_results = []
        for output in results:
            collate_results.append(np.concatenate(output))
    elif isinstance(batch, dict):
        results = list(batch.values())[:len(list(batch.values()))]
        collate_results = []
        for output in results:
            collate_results.append(np.concatenate(output))
    elif isinstance(batch, np.ndarray):
        collate_results = np.concatenate(results)
    else:
        collate_results = results
    return collate_results



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Optimization class."""

import os
from typing import List

from neural_compressor.ux.components.optimization.graph_optimizer.optimize_model import (
    optimize_graph,
    optimize_graph_config,
)
from neural_compressor.ux.components.optimization.optimization import Optimization


class GraphOptimization(Optimization):
    """Optimization class."""

    def execute(self) -> None:
        """Execute graph optimization."""
        if self.tune:
            optimize_graph_config(
                input_graph=self.input_graph,
                output_graph=self.output_graph,
                config=self.config_path,
                framework=self.framework,
            )
        else:
            optimize_graph(
                input_graph=self.input_graph,
                output_graph=self.output_graph,
                input=self.input_nodes,
                output=self.output_nodes,
                framework=self.framework,
                precisions=self.output_precision,
            )

    @property
    def optimization_script(self) -> str:
        """Get optimization script path."""
        return os.path.join(
            os.path.dirname(__file__),
            "optimize_model.py",
        )

    @property
    def parameters(self) -> List[str]:
        """Get optimization parameters."""
        parameters = [
            f"--input-graph={self.input_graph}",
            f"--output-graph={self.output_graph}",
            f"--framework={self.framework}",
        ]

        if self.tune:
            parameters.extend(
                [
                    f"--config={self.config_path}",
                ],
            )
        else:
            parameters.extend(
                [
                    f"--input-nodes={self.input_nodes}",
                    f"--output-nodes={self.output_nodes}",
                    f"--precisions={self.output_precision}",
                ],
            )
        return parameters



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Workloads list test."""

import os
import unittest

from neural_compressor.ux.utils.utils import _load_json_as_dict
from neural_compressor.ux.utils.workload.workloads_list import WorkloadsListMigrator


class TestWorkloadsListMigrator(unittest.TestCase):
    """Workloads list migrator tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Workloads list migrator test constructor."""
        super().__init__(*args, **kwargs)

    def test_clean_environment_migration(self) -> None:
        """Test migration of clean environment."""
        migrator = WorkloadsListMigrator()
        migrator.workloads_json = os.path.join(
            os.path.dirname(__file__),
            "files",
            "non_existing_workloads_list.json",
        )
        self.assertFalse(migrator.require_migration)

    def test_migration_from_v1(self) -> None:
        """Test Workloads List Migration from v1."""
        self._assert_migrates(
            "workloads_list_v1.json",
            "workloads_list_v3.json",
        )

    def test_not_tuned_migration_from_v1(self) -> None:
        """Test Workloads List Migration from v1."""
        self._assert_migrates(
            "workloads_list_v1_not_tuned.json",
            "workloads_list_v3_not_tuned.json",
        )

    def test_empty_migration_from_v1(self) -> None:
        """Test Workloads List Migration from v1."""
        self._assert_migrates(
            "workloads_list_v1_empty.json",
            "workloads_list_v3_empty.json",
        )

    def test_migration_from_v2(self) -> None:
        """Test Workloads List Migration from v2."""
        self._assert_migrates(
            "workloads_list_v2.json",
            "workloads_list_v3.json",
        )

    def test_not_tuned_migration_from_v2(self) -> None:
        """Test Workloads List Migration from v2."""
        self._assert_migrates(
            "workloads_list_v2_not_tuned.json",
            "workloads_list_v3_not_tuned.json",
        )

    def test_empty_migration_from_v2(self) -> None:
        """Test Workloads List Migration from v2."""
        self._assert_migrates(
            "workloads_list_v2_empty.json",
            "workloads_list_v3_empty.json",
        )

    def test_migration_from_v3(self) -> None:
        """Test Workloads List Migration from v3."""
        self._assert_doesnt_migrate("workloads_list_v3.json")

    def test_not_tuned_migration_from_v3(self) -> None:
        """Test Workloads List Migration from v3."""
        self._assert_doesnt_migrate("workloads_list_v3_not_tuned.json")

    def test_empty_migration_from_v3(self) -> None:
        """Test Workloads List Migration from v3."""
        self._assert_doesnt_migrate("workloads_list_v3_empty.json")

    def _assert_migrates(self, initial_file: str, final_file: str) -> None:
        """Test Workloads List Migration from initial to final version."""
        self._assert_migration_works(initial_file, final_file, True)

    def _assert_doesnt_migrate(self, initial_file: str) -> None:
        """Test Workloads List Migration doesn't migrate final version."""
        self._assert_migration_works(initial_file, initial_file, False)

    def _assert_migration_works(
        self,
        initial_file: str,
        final_file: str,
        migration_needed: bool,
    ) -> None:
        """Test Workloads List Migration from initial to final version."""
        migrator = WorkloadsListMigrator()
        migrator.workloads_json = os.path.join(
            os.path.dirname(__file__),
            "files",
            initial_file,
        )

        self.assertEqual(migrator.require_migration, migration_needed)

        migrator.migrate()

        self.assertFalse(migrator.require_migration)

        expected_json_path = os.path.join(
            os.path.dirname(__file__),
            "files",
            final_file,
        )
        expected = _load_json_as_dict(expected_json_path)
        self.assertEqual(type(migrator.workloads_data), dict)
        self.assertDictEqual(migrator.workloads_data, expected)  # type: ignore


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Evaluation config test."""

import unittest
from unittest.mock import MagicMock, patch

from neural_compressor.ux.utils.workload.evaluation import (
    Accuracy,
    Configs,
    Evaluation,
    Metric,
    Performance,
    Postprocess,
    PostprocessSchema,
)


class TestMetricConfig(unittest.TestCase):
    """Metric config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Metric config test constructor."""
        super().__init__(*args, **kwargs)

    def test_metric_constructor(self) -> None:
        """Test Metric config constructor."""
        data = {
            "name": "topk",
            "param": 1,
        }
        metric = Metric(data)

        self.assertEqual(metric.name, "topk")
        self.assertEqual(metric.param, 1)

    def test_metric_constructor_from_yaml(self) -> None:
        """Test Metric config constructor."""
        data = {"topk": 1}
        metric = Metric(data)

        self.assertEqual(metric.name, "topk")
        self.assertEqual(metric.param, 1)

    def test_metric_constructor_defaults(self) -> None:
        """Test Metric config constructor defaults."""
        metric = Metric()

        self.assertIsNone(metric.name)
        self.assertIsNone(metric.param)

    def test_metric_setters(self) -> None:
        """Test metric config setters."""
        metric = Metric()
        metric.name = "topk"
        metric.param = 1

        self.assertEqual(metric.name, "topk")
        self.assertEqual(metric.param, 1)

    def test_metric_setters_with_empty_params(self) -> None:
        """Test metric config setters with empty parameters."""
        metric = Metric()
        metric.name = "TestMetric"
        metric.param = ""

        self.assertEqual(metric.name, "TestMetric")
        self.assertIsNone(metric.param)

    def test_metric_serializer(self) -> None:
        """Test Metric config serializer."""
        data = {
            "name": "topk",
            "param": 1,
        }
        metric = Metric(data)
        result = metric.serialize()

        self.assertDictEqual(
            result,
            {
                "topk": 1,
            },
        )

    def test_MSE_RMSE_MAE_metric_serializer(self) -> None:
        """Test MSE, RMSE and MAE metric config serializer."""
        for metric_name in ["MSE", "RMSE", "MAE"]:
            data = {
                "name": metric_name,
                "param": True,
            }
            metric = Metric(data)
            result = metric.serialize()

            self.assertDictEqual(
                result,
                {
                    f"{metric_name}": {"compare_label": True},
                },
            )

    def test_COCOmAP_metric_serializer_with_anno_path_param(self) -> None:
        """Test COCOmAP metric config serializer."""
        for metric_name in ["COCOmAP"]:
            data = {
                "name": metric_name,
                "param": "/foo/bar/baz",
            }
            metric = Metric(data)
            result = metric.serialize()

            self.assertDictEqual(
                result,
                {
                    f"{metric_name}": {"anno_path": "/foo/bar/baz"},
                },
            )

    def test_COCOmAP_metric_serializer_without_anno_path_param(self) -> None:
        """Test COCOmAP metric config serializer."""
        for metric_name in ["COCOmAP"]:
            data = {
                "name": metric_name,
                "param": {},
            }
            metric = Metric(data)
            result = metric.serialize()
            self.assertDictEqual(
                result,
                {
                    f"{metric_name}": {},
                },
            )

    def test_unnamed_metric_serializer(self) -> None:
        """Test unnamed metric config serializer."""
        data = {
            "param": 1,
            "other param": 2,
        }
        metric = Metric(data)
        result = metric.serialize()

        self.assertDictEqual(result, {})

    def test_COCOmAP_yaml_style_with_params(self) -> None:
        """Test metric with already defined params defaults in yaml style."""
        data = {"COCOmAP": {"anno_path": "/path/to/annotation"}}
        metric = Metric(data)
        self.assertDictEqual(
            metric.serialize(),
            data,
        )

    def test_COCOmAP_yaml_style_without_params(self) -> None:
        """Test metric without defined params defaults in yaml style."""
        data = {"COCOmAP": {}}
        metric = Metric(data)
        self.assertDictEqual(
            metric.serialize(),
            data,
        )

    def test_MSE_RMSE_MAE_metric_yaml_style(self) -> None:
        """Test MSE, RMSE and MAE metrics serialization in yaml style."""
        for metric_name in ["MSE", "RMSE", "MAE"]:
            data = {
                f"{metric_name}": {
                    "compare_label": True,
                },
            }
            metric = Metric(data)
            result = metric.serialize()

            self.assertDictEqual(
                result,
                {
                    f"{metric_name}": {"compare_label": True},
                },
            )


class TestConfigsConfig(unittest.TestCase):
    """Configs config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Configs config test constructor."""
        super().__init__(*args, **kwargs)

    @patch("neural_compressor.ux.utils.workload.evaluation.HWInfo")
    def test_configs_constructor(self, mock_hwinfo: MagicMock) -> None:
        """Test Configs config constructor."""
        mock_hwinfo.return_value.cores_per_socket = 5
        mock_hwinfo.return_value.sockets = 3

        data = {
            "cores_per_instance": 2,
            "num_of_instance": 4,
            "inter_num_of_threads": 8,
            "intra_num_of_threads": 16,
            "kmp_blocktime": 3,
        }
        configs = Configs(data)

        self.assertEqual(configs.cores_per_instance, 2)
        self.assertEqual(configs.num_of_instance, 4)
        self.assertEqual(configs.inter_num_of_threads, 8)
        self.assertEqual(configs.intra_num_of_threads, 16)
        self.assertEqual(configs.kmp_blocktime, 3)

    @patch("neural_compressor.ux.utils.workload.evaluation.HWInfo")
    def test_configs_constructor_defaults(self, mock_hwinfo: MagicMock) -> None:
        """Test Configs config constructor defaults."""
        mock_hwinfo.return_value.cores = 5
        mock_hwinfo.return_value.sockets = 3

        configs = Configs()

        self.assertEqual(configs.cores_per_instance, 4)
        self.assertEqual(configs.num_of_instance, 1)
        self.assertIsNone(configs.inter_num_of_threads)
        self.assertIsNone(configs.intra_num_of_threads)
        self.assertEqual(1, configs.kmp_blocktime)

    @patch("neural_compressor.ux.utils.workload.evaluation.HWInfo")
    def test_configs_serializer(self, mock_hwinfo: MagicMock) -> None:
        """Test Configs config serializer."""
        mock_hwinfo.return_value.cores_per_socket = 5
        mock_hwinfo.return_value.sockets = 3

        data = {
            "cores_per_instance": 2,
            "num_of_instance": 4,
            "inter_num_of_threads": 8,
            "intra_num_of_threads": 16,
            "kmp_blocktime": 3,
        }
        configs = Configs(data)
        result = configs.serialize()

        self.assertDictEqual(
            result,
            {
                "cores_per_instance": 2,
                "num_of_instance": 4,
                "inter_num_of_threads": 8,
                "intra_num_of_threads": 16,
                "kmp_blocktime": 3,
            },
        )


class TestPostprocessSchemaConfig(unittest.TestCase):
    """PostprocessSchema config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Postprocess Schema config test constructor."""
        super().__init__(*args, **kwargs)

    def test_postprocess_schema_constructor(self) -> None:
        """Test PostprocessSchema config constructor."""
        data = {
            "LabelShift": 1,
            "SquadV1": {
                "label_file": "/path/to/dev-v1.1.json",
                "vocab_file": "/path/to/vocab.txt",
            },
        }
        postprocess_schema = PostprocessSchema(data)

        self.assertEqual(postprocess_schema.LabelShift, 1)
        self.assertDictEqual(
            postprocess_schema.SquadV1,
            {
                "label_file": "/path/to/dev-v1.1.json",
                "vocab_file": "/path/to/vocab.txt",
            },
        )

    def test_postprocess_schema_constructor_defaults(self) -> None:
        """Test PostprocessSchema config constructor defaults."""
        postprocess_schema = PostprocessSchema()

        self.assertIsNone(postprocess_schema.LabelShift)
        self.assertIsNone(postprocess_schema.SquadV1)


class TestPostprocessConfig(unittest.TestCase):
    """Postprocess config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Postprocess config test constructor."""
        super().__init__(*args, **kwargs)

    def test_postprocess_constructor(self) -> None:
        """Test Postprocess config constructor."""
        data = {
            "transform": {
                "LabelShift": 1,
                "SquadV1": {
                    "label_file": "/path/to/dev-v1.1.json",
                    "vocab_file": "/path/to/vocab.txt",
                },
            },
        }
        postprocess = Postprocess(data)

        self.assertIsNotNone(postprocess.transform)
        self.assertEqual(postprocess.transform.LabelShift, 1)
        self.assertDictEqual(
            postprocess.transform.SquadV1,
            {
                "label_file": "/path/to/dev-v1.1.json",
                "vocab_file": "/path/to/vocab.txt",
            },
        )

    def test_postprocess_constructor_defaults(self) -> None:
        """Test Postprocess config constructor defaults."""
        postprocess = Postprocess()

        self.assertIsNone(postprocess.transform)


class TestAccuracyConfig(unittest.TestCase):
    """Accuracy config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Accuracy config test constructor."""
        super().__init__(*args, **kwargs)

    @patch("neural_compressor.ux.utils.workload.evaluation.HWInfo")
    def test_accuracy_constructor(self, mock_hwinfo: MagicMock) -> None:
        """Test Accuracy config constructor."""
        mock_hwinfo.return_value.cores_per_socket = 5
        mock_hwinfo.return_value.sockets = 3
        data = {
            "metric": {"topk": 1},
            "configs": {
                "cores_per_instance": 2,
                "num_of_instance": 4,
                "inter_num_of_threads": 8,
                "intra_num_of_threads": 16,
                "kmp_blocktime": 3,
            },
            "dataloader": {
                "last_batch": "rollover",
                "batch_size": 2,
                "dataset": {
                    "TestDataset": {
                        "dataset_param": "/some/path",
                        "bool_param": True,
                        "list_param": ["item1", "item2"],
                    },
                },
                "transform": {
                    "TestTransform": {"shape": [1000, 224, 224, 3], "some_op": True},
                    "AnotherTestTransform": {
                        "shape": [10, 299, 299, 3],
                        "some_op": False,
                    },
                },
                "filter": {
                    "LabelBalance": {"size": 1},
                },
            },
            "postprocess": {
                "transform": {
                    "LabelShift": 1,
                    "SquadV1": {
                        "label_file": "/path/to/dev-v1.1.json",
                        "vocab_file": "/path/to/vocab.txt",
                    },
                },
            },
        }
        accuracy = Accuracy(data)

        self.assertIsNotNone(accuracy.metric)
        self.assertEqual(accuracy.metric.name, "topk")
        self.assertEqual(accuracy.metric.param, 1)

        self.assertIsNotNone(accuracy.configs)
        self.assertEqual(
            accuracy.configs.cores_per_instance,
            2,
        )
        self.assertEqual(
            accuracy.configs.num_of_instance,
            4,
        )
        self.assertEqual(accuracy.configs.inter_num_of_threads, 8)
        self.assertEqual(accuracy.configs.intra_num_of_threads, 16)
        self.assertEqual(accuracy.configs.kmp_blocktime, 3)

        self.assertIsNotNone(accuracy.dataloader)
        self.assertEqual(accuracy.dataloader.last_batch, "rollover")
        self.assertEqual(
            accuracy.dataloader.batch_size,
            2,
        )
        self.assertIsNotNone(accuracy.dataloader.dataset)
        self.assertEqual(accuracy.dataloader.dataset.name, "TestDataset")
        self.assertDictEqual(
            accuracy.dataloader.dataset.params,
            {
                "dataset_param": "/some/path",
                "bool_param": True,
                "list_param": ["item1", "item2"],
            },
        )
        transform_name, transform = list(accuracy.dataloader.transform.items())[0]
        self.assertEqual(transform_name, "TestTransform")
        self.assertEqual(transform.name, "TestTransform")
        self.assertDictEqual(
            transform.parameters,
            {
                "shape": [1000, 224, 224, 3],
                "some_op": True,
            },
        )
        transform_name, transform = list(accuracy.dataloader.transform.items())[1]
        self.assertEqual(transform_name, "AnotherTestTransform")
        self.assertEqual(transform.name, "AnotherTestTransform")
        self.assertDictEqual(
            transform.parameters,
            {"shape": [10, 299, 299, 3], "some_op": False},
        )
        self.assertIsNotNone(accuracy.dataloader.filter)
        self.assertIsNotNone(accuracy.dataloader.filter.LabelBalance)
        self.assertEqual(accuracy.dataloader.filter.LabelBalance.size, 1)

        self.assertIsNotNone(accuracy.postprocess)
        self.assertIsNotNone(accuracy.postprocess.transform)
        self.assertEqual(accuracy.postprocess.transform.LabelShift, 1)
        self.assertDictEqual(
            accuracy.postprocess.transform.SquadV1,
            {
                "label_file": "/path/to/dev-v1.1.json",
                "vocab_file": "/path/to/vocab.txt",
            },
        )

    def test_accuracy_constructor_defaults(self) -> None:
        """Test Accuracy config constructor defaults."""
        accuracy = Accuracy()

        self.assertIsNone(accuracy.metric)
        self.assertIsNone(accuracy.configs)
        self.assertIsNone(accuracy.dataloader)
        self.assertIsNone(accuracy.postprocess)


class TestPerformanceConfig(unittest.TestCase):
    """Performance config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Initialize Performance config test."""
        super().__init__(*args, **kwargs)

    @patch("neural_compressor.ux.utils.workload.evaluation.HWInfo")
    def test_performance_constructor(self, mock_hwinfo: MagicMock) -> None:
        """Test Performance config constructor."""
        mock_hwinfo.return_value.cores_per_socket = 5
        mock_hwinfo.return_value.sockets = 3
        data = {
            "warmup": 100,
            "iteration": 1000,
            "configs": {
                "cores_per_instance": 2,
                "num_of_instance": 4,
                "inter_num_of_threads": 8,
                "intra_num_of_threads": 16,
                "kmp_blocktime": 3,
            },
            "dataloader": {
                "last_batch": "rollover",
                "batch_size": 2,
                "dataset": {
                    "TestDataset": {
                        "dataset_param": "/some/path",
                        "bool_param": True,
                        "list_param": ["item1", "item2"],
                    },
                },
                "transform": {
                    "TestTransform": {"shape": [1000, 224, 224, 3], "some_op": True},
                    "AnotherTestTransform": {
                        "shape": [10, 299, 299, 3],
                        "some_op": False,
                    },
                },
                "filter": {
                    "LabelBalance": {"size": 1},
                },
            },
            "postprocess": {
                "transform": {
                    "LabelShift": 1,
                    "SquadV1": {
                        "label_file": "/path/to/dev-v1.1.json",
                        "vocab_file": "/path/to/vocab.txt",
                    },
                },
            },
        }
        performance = Performance(data)

        self.assertEqual(performance.warmup, 100)
        self.assertEqual(performance.iteration, 1000)

        self.assertIsNotNone(performance.configs)
        self.assertEqual(
            performance.configs.cores_per_instance,
            2,
        )
        self.assertEqual(
            performance.configs.num_of_instance,
            4,
        )
        self.assertEqual(performance.configs.inter_num_of_threads, 8)
        self.assertEqual(performance.configs.intra_num_of_threads, 16)
        self.assertEqual(performance.configs.kmp_blocktime, 3)

        self.assertIsNotNone(performance.dataloader)
        self.assertEqual(performance.dataloader.last_batch, "rollover")
        self.assertEqual(
            performance.dataloader.batch_size,
            2,
        )
        self.assertIsNotNone(performance.dataloader.dataset)
        self.assertEqual(performance.dataloader.dataset.name, "TestDataset")
        self.assertDictEqual(
            performance.dataloader.dataset.params,
            {
                "dataset_param": "/some/path",
                "bool_param": True,
                "list_param": ["item1", "item2"],
            },
        )
        transform_name, transform = list(performance.dataloader.transform.items())[0]
        self.assertEqual(transform_name, "TestTransform")
        self.assertEqual(transform.name, "TestTransform")
        self.assertDictEqual(
            transform.parameters,
            {
                "shape": [1000, 224, 224, 3],
                "some_op": True,
            },
        )
        transform_name, transform = list(performance.dataloader.transform.items())[1]
        self.assertEqual(transform_name, "AnotherTestTransform")
        self.assertEqual(transform.name, "AnotherTestTransform")
        self.assertDictEqual(
            transform.parameters,
            {"shape": [10, 299, 299, 3], "some_op": False},
        )
        self.assertIsNotNone(performance.dataloader.filter)
        self.assertIsNotNone(performance.dataloader.filter.LabelBalance)
        self.assertEqual(performance.dataloader.filter.LabelBalance.size, 1)

        self.assertIsNotNone(performance.postprocess)
        self.assertIsNotNone(performance.postprocess.transform)
        self.assertEqual(performance.postprocess.transform.LabelShift, 1)
        self.assertDictEqual(
            performance.postprocess.transform.SquadV1,
            {
                "label_file": "/path/to/dev-v1.1.json",
                "vocab_file": "/path/to/vocab.txt",
            },
        )

    @patch("neural_compressor.ux.utils.workload.evaluation.HWInfo")
    def test_performance_constructor_defaults(self, mock_hwinfo: MagicMock) -> None:
        """Test Performance config constructor defaults."""
        mock_hwinfo.return_value.cores = 5
        mock_hwinfo.return_value.sockets = 3

        performance = Performance()

        self.assertEqual(performance.warmup, 5)

        self.assertEqual(performance.iteration, -1)

        self.assertIsNotNone(performance.configs)
        self.assertEqual(performance.configs.cores_per_instance, 4)
        self.assertEqual(performance.configs.num_of_instance, 1)
        self.assertIsNone(performance.configs.inter_num_of_threads)
        self.assertIsNone(performance.configs.intra_num_of_threads)
        self.assertEqual(1, performance.configs.kmp_blocktime)

        self.assertIsNone(performance.dataloader)

        self.assertIsNone(performance.postprocess)


class TestEvaluationConfig(unittest.TestCase):
    """Evaluation config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Initialize Evaluation config test."""
        super().__init__(*args, **kwargs)

    @patch("neural_compressor.ux.utils.workload.evaluation.HWInfo")
    def test_evaluation_constructor(self, mock_hwinfo: MagicMock) -> None:
        """Test Evaluation config constructor."""
        mock_hwinfo.return_value.cores_per_socket = 5
        mock_hwinfo.return_value.sockets = 3
        data = {
            "accuracy": {
                "metric": {"topk": 1},
                "configs": {
                    "cores_per_instance": 2,
                    "num_of_instance": 4,
                    "inter_num_of_threads": 8,
                    "intra_num_of_threads": 16,
                    "kmp_blocktime": 3,
                },
                "dataloader": {
                    "last_batch": "rollover",
                    "batch_size": 2,
                    "dataset": {
                        "TestDataset": {
                            "dataset_param": "/some/path",
                            "bool_param": True,
                            "list_param": ["item1", "item2"],
                        },
                    },
                    "transform": {
                        "TestTransform": {
                            "shape": [1000, 224, 224, 3],
                            "some_op": True,
                        },
                        "AnotherTestTransform": {
                            "shape": [10, 299, 299, 3],
                            "some_op": False,
                        },
                    },
                    "filter": {
                        "LabelBalance": {"size": 1},
                    },
                },
                "postprocess": {
                    "transform": {
                        "LabelShift": 1,
                        "SquadV1": {
                            "label_file": "/path/to/dev-v1.1.json",
                            "vocab_file": "/path/to/vocab.txt",
                        },
                    },
                },
            },
            "performance": {
                "warmup": 100,
                "iteration": 1000,
                "configs": {
                    "cores_per_instance": 2,
                    "num_of_instance": 4,
                    "inter_num_of_threads": 8,
                    "intra_num_of_threads": 16,
                    "kmp_blocktime": 3,
                },
                "dataloader": {
                    "last_batch": "rollover",
                    "batch_size": 2,
                    "dataset": {
                        "TestDataset": {
                            "dataset_param": "/some/path",
                            "bool_param": True,
                            "list_param": ["item1", "item2"],
                        },
                    },
                    "transform": {
                        "TestTransform": {
                            "shape": [1000, 224, 224, 3],
                            "some_op": True,
                        },
                        "AnotherTestTransform": {
                            "shape": [10, 299, 299, 3],
                            "some_op": False,
                        },
                    },
                    "filter": {
                        "LabelBalance": {"size": 1},
                    },
                },
                "postprocess": {
                    "transform": {
                        "LabelShift": 1,
                        "SquadV1": {
                            "label_file": "/path/to/dev-v1.1.json",
                            "vocab_file": "/path/to/vocab.txt",
                        },
                    },
                },
            },
        }
        evaluation = Evaluation(data)

        self.assertIsNotNone(evaluation.accuracy)
        self.assertIsNotNone(evaluation.accuracy.metric)
        self.assertEqual(evaluation.accuracy.metric.name, "topk")
        self.assertEqual(evaluation.accuracy.metric.param, 1)

        self.assertIsNotNone(evaluation.accuracy.configs)
        self.assertEqual(
            evaluation.accuracy.configs.cores_per_instance,
            2,
        )
        self.assertEqual(
            evaluation.accuracy.configs.num_of_instance,
            4,
        )
        self.assertEqual(evaluation.accuracy.configs.inter_num_of_threads, 8)
        self.assertEqual(evaluation.accuracy.configs.intra_num_of_threads, 16)
        self.assertEqual(evaluation.accuracy.configs.kmp_blocktime, 3)

        self.assertIsNotNone(evaluation.accuracy.dataloader)
        self.assertEqual(evaluation.accuracy.dataloader.last_batch, "rollover")
        self.assertEqual(
            evaluation.accuracy.dataloader.batch_size,
            2,
        )
        self.assertIsNotNone(evaluation.accuracy.dataloader.dataset)
        self.assertEqual(evaluation.accuracy.dataloader.dataset.name, "TestDataset")
        self.assertDictEqual(
            evaluation.accuracy.dataloader.dataset.params,
            {
                "dataset_param": "/some/path",
                "bool_param": True,
                "list_param": ["item1", "item2"],
            },
        )
        transform_name, transform = list(
            evaluation.accuracy.dataloader.transform.items(),
        )[0]
        self.assertEqual(transform_name, "TestTransform")
        self.assertEqual(transform.name, "TestTransform")
        self.assertDictEqual(
            transform.parameters,
            {
                "shape": [1000, 224, 224, 3],
                "some_op": True,
            },
        )
        transform_name, transform = list(
            evaluation.accuracy.dataloader.transform.items(),
        )[1]
        self.assertEqual(transform_name, "AnotherTestTransform")
        self.assertEqual(transform.name, "AnotherTestTransform")
        self.assertDictEqual(
            transform.parameters,
            {"shape": [10, 299, 299, 3], "some_op": False},
        )
        self.assertIsNotNone(evaluation.accuracy.dataloader.filter)
        self.assertIsNotNone(evaluation.accuracy.dataloader.filter.LabelBalance)
        self.assertEqual(evaluation.accuracy.dataloader.filter.LabelBalance.size, 1)

        self.assertIsNotNone(evaluation.accuracy.postprocess)
        self.assertIsNotNone(evaluation.accuracy.postprocess.transform)
        self.assertEqual(evaluation.accuracy.postprocess.transform.LabelShift, 1)
        self.assertDictEqual(
            evaluation.accuracy.postprocess.transform.SquadV1,
            {
                "label_file": "/path/to/dev-v1.1.json",
                "vocab_file": "/path/to/vocab.txt",
            },
        )

        self.assertIsNotNone(evaluation.performance)
        self.assertEqual(evaluation.performance.warmup, 100)
        self.assertEqual(evaluation.performance.iteration, 1000)

        self.assertIsNotNone(evaluation.performance.configs)
        self.assertEqual(
            evaluation.performance.configs.cores_per_instance,
            2,
        )
        self.assertEqual(
            evaluation.performance.configs.num_of_instance,
            4,
        )
        self.assertEqual(evaluation.performance.configs.inter_num_of_threads, 8)
        self.assertEqual(evaluation.performance.configs.intra_num_of_threads, 16)
        self.assertEqual(evaluation.performance.configs.kmp_blocktime, 3)

        self.assertIsNotNone(evaluation.performance.dataloader)
        self.assertEqual(evaluation.performance.dataloader.last_batch, "rollover")
        self.assertEqual(
            evaluation.performance.dataloader.batch_size,
            2,
        )
        self.assertIsNotNone(evaluation.performance.dataloader.dataset)
        self.assertEqual(
            evaluation.performance.dataloader.dataset.name,
            "TestDataset",
        )
        self.assertDictEqual(
            evaluation.performance.dataloader.dataset.params,
            {
                "dataset_param": "/some/path",
                "bool_param": True,
                "list_param": ["item1", "item2"],
            },
        )
        transform_name, transform = list(
            evaluation.performance.dataloader.transform.items(),
        )[0]
        self.assertEqual(transform_name, "TestTransform")
        self.assertEqual(transform.name, "TestTransform")
        self.assertDictEqual(
            transform.parameters,
            {
                "shape": [1000, 224, 224, 3],
                "some_op": True,
            },
        )
        transform_name, transform = list(
            evaluation.performance.dataloader.transform.items(),
        )[1]
        self.assertEqual(transform_name, "AnotherTestTransform")
        self.assertEqual(transform.name, "AnotherTestTransform")
        self.assertDictEqual(
            transform.parameters,
            {"shape": [10, 299, 299, 3], "some_op": False},
        )
        self.assertIsNotNone(evaluation.performance.dataloader.filter)
        self.assertIsNotNone(evaluation.performance.dataloader.filter.LabelBalance)
        self.assertEqual(
            evaluation.performance.dataloader.filter.LabelBalance.size,
            1,
        )

        self.assertIsNotNone(evaluation.performance.postprocess)
        self.assertIsNotNone(evaluation.performance.postprocess.transform)
        self.assertEqual(evaluation.performance.postprocess.transform.LabelShift, 1)
        self.assertDictEqual(
            evaluation.performance.postprocess.transform.SquadV1,
            {
                "label_file": "/path/to/dev-v1.1.json",
                "vocab_file": "/path/to/vocab.txt",
            },
        )

    def test_evaluation_constructor_defaults(self) -> None:
        """Test Evaluation config constructor defaults."""
        evaluation = Evaluation()

        self.assertIsNone(evaluation.accuracy)
        self.assertIsNone(evaluation.performance)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
1. First use model loader to get the computation graph with corresponding framework.
   The graph contains nodes and edges, the node is op and the edge is the tensor.
2. Then extract the ops in the graph and pack them to our form.
3. Next exploit these above ops to consist sub-graph, which can see as "a new big op", like
   LayerNorm. Note that there may have different computation flow in one subgraph.
4. Finally, convert them to .yaml file and .bin file for model configuration and inference.
"""

import datetime
import logging as log
import os
import sys
import traceback
from collections import OrderedDict
from .loaders.loader import Loader
from .extractors.extractor import Extractor
from .sub_graph.subgraph_matcher import SubGraphMatcher


CONVERTERS = OrderedDict({
    'loader': Loader,
    'extractor': Extractor,
    'sub_graph': SubGraphMatcher,
})


def start_pipeline(model, config=None):
    converter_list = []
    # initialize the converter
    for converter_type in CONVERTERS.keys():
        converter = CONVERTERS[converter_type]()
        converter_list.append(converter)
    # convert the model
    for converter in converter_list:
        model = converter(model)
    return model


def prepare_ir(model, config=None):
    model = start_pipeline(model, config=None)
    return model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tensorflow.python.framework import tensor_util
import tensorflow as tf
from neural_compressor.utils import logger
import numpy as np
from ..graph.graph import Graph
from ..ops.op import OPERATORS
from ..tf_utils import graph_node_names_details
from ..graph_utils import names_from_input


class TensorflowExtractor(object):
    """
    Decorate the node in model.graph_def, and the new node has the attributes like input_tensors
    and output_tensors, these tensors record the source/dest op name. All of these nodes
    (in a list) will compose a graph, which is Graph class, as the return object.
    Args:
        model: neural_compressor TensorflowBaseModel
    Return:
        Graph: Graph class, the new graph object

    """
    @classmethod
    def __call__(self, model):

        nodes = model.graph_def.node
        graph_nodes_dict = graph_node_names_details(nodes)
        logger.info('Start to extarct tensorflow model ops...')
        new_graph = Graph()
        new_graph.framework = 'tensorflow'
        for node in nodes:
            # ignore the Const nodes, they have no source inputs
            if node.op == 'Const':
                continue
            else:
                op_type = node.op
                if op_type in OPERATORS.keys():
                    new_node = OPERATORS[op_type]()
                    new_node.extract('tensorflow', node, model, graph_nodes_dict)
                    new_graph.insert_nodes(len(new_graph.nodes), [new_node])
                else:
                    raise ValueError('the {} operation does not support now...'.format(op_type))

        return new_graph



import os
import unittest
import yaml
import shutil
import tensorflow as tf
from tensorflow.python.framework import graph_util
from neural_compressor.adaptor.tensorflow import TensorFlowAdaptor
from neural_compressor.adaptor.tf_utils.util import disable_random

def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            accuracy_criterion:
              relative: 0.0001
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()
class TestQuantizeInput(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        build_fake_yaml()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml.yaml')
        shutil.rmtree('./saved', ignore_errors=True)

    @disable_random()
    @unittest.skipIf(tf.version.VERSION < '2.1.0', "Quantize input needs tensorflow 2.1.0 and newer, so test_quantize_input is skipped")
    def test_quantize_input(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(x, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")

        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())

        conv_bias = tf.math.add(conv, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])

            for i in constant_graph.node:
                if i.op.find('Add') != -1:
                    i.op = 'Add'

            from neural_compressor.experimental import Quantization, common
            quantizer = Quantization("./fake_yaml.yaml")
            dataset = quantizer.dataset('dummy', shape=(100, 56, 56, 16), label=True)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.model = constant_graph
            q_model = quantizer()

            framework_specific_info = {'device': 'cpu', 'approach': 'post_training_static_quant', \
                'random_seed': 1978, 'inputs': ['input'], 'outputs': ['op_to_store'], \
                    'workspace_path': 'saved'}

            quantize_input_graph, _ = TensorFlowAdaptor(framework_specific_info).quantize_input(q_model.graph)
            Not_found_QuantizedV2 = True
            for i in quantize_input_graph.as_graph_def().node:
                if i.op == 'QuantizeV2':
                    Not_found_QuantizedV2 = False
                    break
            self.assertEqual(Not_found_QuantizedV2, True)


if __name__ == "__main__":
    unittest.main()



#
#  -*- coding: utf-8 -*-
#
import unittest
import yaml
import os
import tensorflow as tf


from neural_compressor.adaptor.tensorflow import TensorflowQuery
from neural_compressor.adaptor.tf_utils.util import disable_random
from tensorflow.python.framework import graph_util


def build_fake_yaml_on_grappler():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        device: cpu
        quantization:
          recipes:
            scale_propagation_concat: False
          model_wise:
            weight:
                granularity: per_tensor
                scheme: sym
                dtype: int8
                algorithm: minmax
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            strategy:
              name: basic
            accuracy_criterion:
              relative: 0.1
            exit_policy:
              performance_only: True
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml_grappler.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

class TestTFQueryYaml(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.tf_yaml_path = os.path.join(os.getcwd() + "/../neural_compressor/adaptor/tensorflow.yaml")

        with open(self.tf_yaml_path) as f:
            self.content = yaml.safe_load(f)
        self.query_handler = TensorflowQuery(local_config_file=self.tf_yaml_path)
        build_fake_yaml_on_grappler()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml_grappler.yaml')

    def test_unique_version(self):
        versions = [i['version']['name'] for i in self.content]
        registered_version_name = []
        for i in versions:
          if isinstance(i, list):
            registered_version_name.extend(i)
          else:
            registered_version_name.append(i)

        self.assertEqual(len(registered_version_name), len(set(registered_version_name)))

    def test_int8_sequences(self):
        patterns = self.query_handler.get_eightbit_patterns()

        has_conv2d = bool('Conv2D' in patterns)
        has_matmul = bool('MatMul' in patterns)
        self.assertEqual(has_conv2d, True)
        self.assertEqual(has_matmul, True)
        self.assertGreaterEqual(len(patterns['Conv2D']), 13)
        self.assertGreaterEqual(len(patterns['MatMul']), 3)
        self.assertEqual(len(patterns['ConcatV2']), 1)
        self.assertEqual(len(patterns['MaxPool']), 1)
        self.assertEqual(len(patterns['AvgPool']), 1)

    def test_convert_internal_patterns(self):
        internal_patterns = self.query_handler.generate_internal_patterns()
        self.assertEqual([['MaxPool']] in internal_patterns, True)
        self.assertEqual([['ConcatV2']] in internal_patterns, True)
        self.assertEqual([['AvgPool']] in internal_patterns, True)
        self.assertEqual([['MatMul'], ('BiasAdd',), ('Relu',)] in internal_patterns, True)

    @disable_random()
    def test_grappler_cfg(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 30, 30, 1], name="input")
        conv_weights = tf.compat.v1.get_variable("weight", [2, 2, 1, 1],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_bias = tf.compat.v1.get_variable("bias", [1],
                                              initializer=tf.compat.v1.random_normal_initializer())

        x = tf.nn.relu(x)
        conv = tf.nn.conv2d(x, conv_weights, strides=[1, 2, 2, 1], padding="SAME", name='last')
        normed = tf.compat.v1.layers.batch_normalization(conv)

        relu = tf.nn.relu(normed)
        relu2 = tf.nn.relu(relu)
        pool = tf.nn.max_pool(relu2, ksize=1, strides=[1, 2, 2, 1], name='maxpool', padding="SAME")
        conv1 = tf.nn.conv2d(pool, conv_weights, strides=[1, 2, 2, 1], padding="SAME", name='last')
        conv_bias = tf.nn.bias_add(conv1, conv_bias)
        x = tf.nn.relu(conv_bias)
        final_node = tf.nn.relu(x, name='op_to_store')

        out_name = final_node.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Quantization, common

            quantizer = Quantization('fake_yaml_grappler.yaml')
            dataset = quantizer.dataset('dummy', shape=(100, 30, 30, 1), label=True)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.eval_dataloader = common.DataLoader(dataset)
            quantizer.model = output_graph_def
            output_graph = quantizer()

            disable_arithmetic = False
            for i in output_graph.graph_def.node:
              if i.name == 'maxpool_eightbit_quantize_Relu_2' and i.input[0] == 'Relu_2':
                disable_arithmetic = True
            # if tf.version.VERSION >= '2.3.0':
            #     self.assertEqual(False, disable_arithmetic)
            # else:
            self.assertEqual(True, disable_arithmetic)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neural_compressor.utils.utility import LazyImport
from .base_dataloader import BaseDataLoader
from .default_dataloader import DefaultDataLoader
import numpy as np
import collections
import logging

def default_collate(batch):
    elem = batch[0]
    if isinstance(elem, collections.abc.Sequence):
        batch = zip(*batch)
        return [default_collate(samples) for samples in batch]
    elif isinstance(elem, dict):
        return elem
    else:
        try:
            return np.stack(batch)
        except:
            return batch

class EngineDataLoader(BaseDataLoader):
    def _generate_dataloader(self, dataset, batch_size, last_batch, collate_fn,
                             sampler, batch_sampler, num_workers, pin_memory,
                             shuffle, distributed):
        if shuffle:
            logging.warning('Shuffle is not supported yet in EngineDataLoader, ' \
                            'ignoring shuffle keyword.')
        collate_fn = default_collate
        return DefaultDataLoader(dataset, batch_size, last_batch, collate_fn,
                                 sampler, batch_sampler, num_workers, pin_memory,
                                 shuffle, distributed)



"""Tests for collecting layer histogram."""
from neural_compressor.utils.collect_layer_histogram import LayerHistogramCollector
from collections import OrderedDict
from neural_compressor.utils import logger
import numpy as np
import torch
import torch.nn as nn
import unittest

class InvertedResidual(nn.Module):
    def __init__(self, inp, oup, stride, expand_ratio):
        super().__init__()
        self.stride = stride
        assert stride in [1, 2]

        hidden_dim = int(inp * expand_ratio)
        self.use_res_connect = self.stride == 1 and inp == oup

        if expand_ratio == 1:
            self.conv = nn.Sequential(
                nn.Conv2d(hidden_dim, hidden_dim, 3, stride, 1, groups=hidden_dim, bias=False),
                nn.BatchNorm2d(hidden_dim),
                nn.ReLU6(inplace=True),
                nn.Conv2d(hidden_dim, oup, 1, 1, 0, bias=False),
                nn.BatchNorm2d(oup),
            )
        else:
            self.conv = nn.Sequential(
                nn.Conv2d(inp, hidden_dim, 1, 1, 0, bias=False),
                nn.BatchNorm2d(hidden_dim),
                nn.ReLU6(inplace=True),
                nn.Conv2d(hidden_dim, hidden_dim, 3, stride, 1, groups=hidden_dim, bias=False),
                nn.BatchNorm2d(hidden_dim),
                nn.ReLU6(inplace=True),
                nn.Conv2d(hidden_dim, oup, 1, 1, 0, bias=False),
                nn.BatchNorm2d(oup),
            )

    def forward(self, x):
        if self.use_res_connect:
            return x + self.conv(x)
        else:
            return self.conv(x)

class BuildFakeModel(nn.Module):
    def conv_1x1_bn(self, inp, oup):
        return nn.Sequential(
            nn.Conv2d(inp, oup, 1, 1, 0, bias=False),
            nn.BatchNorm2d(oup),
            nn.ReLU6(inplace=True)
        )

    def conv_bn(self, inp, oup, stride):
        return nn.Sequential(
            nn.Conv2d(inp, oup, 3, stride, 1, bias=False),
            nn.BatchNorm2d(oup),
            nn.ReLU6(inplace=True)
        )

    def make_divisible(self, x, divisor=8):
        return int(np.ceil(x * 1. / divisor) * divisor)

    def __init__(self, n_class=1000, input_size=224, width_mult=1.):
        super().__init__()
        block = InvertedResidual
        input_channel = 32
        last_channel = 1280
        interverted_residual = [
            [1, 16, 1, 1],
            [6, 24, 2, 2],
            [6, 32, 3, 2],
            [6, 64, 4, 2],
            [6, 96, 3, 1],
            [6, 160, 3, 2],
            [6, 320, 1, 1],
        ]

        assert input_size % 32 == 0
        self.last_channel = self.make_divisible(last_channel * width_mult) if width_mult > 1.0 else last_channel
        self.features = [self.conv_bn(3, input_channel, 2)]
        for t, c, n, s in interverted_residual:
            output_channel = self.make_divisible(c * width_mult) if t > 1 else c
            for i in range(n):
                if i == 0:
                    self.features.append(block(input_channel, output_channel, s, expand_ratio=t))
                else:
                    self.features.append(block(input_channel, output_channel, 1, expand_ratio=t))
                input_channel = output_channel
        self.features.append(self.conv_1x1_bn(input_channel, self.last_channel))
        self.features = nn.Sequential(*self.features)
        self.classifier = nn.Linear(self.last_channel, n_class)

    def forward(self, x):
        x = self.features(x)
        x = torch.mean(x.view(x.size(0), x.size(1), -1), -1)
        x = self.classifier(x)
        return x

class CollectLayerHistogram(unittest.TestCase):
    def setUp(self):
        model = BuildFakeModel(width_mult=1)
        layer_tensor, include_layer = OrderedDict(), OrderedDict()
        i = 0
        for key, value in model.state_dict().items():
            if not value.ndim:
                value = np.expand_dims(value, axis=0)
            if i>200:
                pass
            else:
                include_layer[key] = np.array(value, dtype=np.float32)
            layer_tensor[key] = np.array(value, dtype=np.float32)
            i += 1
        self.layer_histogram_collector = LayerHistogramCollector \
            (num_bins=8001, layer_tensor=layer_tensor, include_layer=include_layer, logger=logger)
        
    def test_layer_histogram(self):
        self.layer_histogram_collector.collect()
        self.assertEqual(self.layer_histogram_collector.layer_tensor.keys() \
            & self.layer_histogram_collector.include_layer.keys(), \
                self.layer_histogram_collector.hist_dict.keys())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""UX server HW info module."""

import platform
import re
import subprocess
from typing import Any, Dict, Union

import cpuinfo
import psutil

from neural_compressor.ux.utils.json_serializer import JsonSerializer
from neural_compressor.ux.utils.logger import log
from neural_compressor.ux.utils.utils import determine_ip


class HWInfo(JsonSerializer):
    """Class responsible for gathering information about platform hardware."""

    initialized: bool = False

    sockets: int = 0
    cores: int = 0
    cores_per_socket: int = 0
    threads_per_socket: int = 0
    total_memory: str = ""
    system: str = ""
    ip: str = ""
    platform: str = ""
    hyperthreading_enabled: bool = False
    turboboost_enabled: Union[str, bool] = False
    bios_version: str = ""
    kernel: str = ""
    min_cpu_freq: str = ""
    max_cpu_freq: str = ""

    def __init__(self) -> None:
        """Initialize HW Info class and gather information platform hardware."""
        super().__init__()
        if not HWInfo.initialized:
            self.initialize()

    def serialize(
        self,
        serialization_type: str = "default",
    ) -> Dict[str, Any]:
        """Serialize class to dict."""
        return {
            "sockets": self.sockets,
            "cores": self.cores,
            "cores_per_socket": self.cores_per_socket,
            "threads_per_socket": self.threads_per_socket,
            "total_memory": self.total_memory,
            "system": self.system,
            "ip": self.ip,
            "platform": self.platform,
            "hyperthreading_enabled": self.hyperthreading_enabled,
            "turboboost_enabled": self.turboboost_enabled,
            "bios_version": self.bios_version,
            "kernel": self.kernel,
            "min_cpu_freq": self.min_cpu_freq,
            "max_cpu_freq": self.max_cpu_freq,
        }

    def initialize(self) -> None:
        """Set class variables."""
        cpu_info = cpuinfo.get_cpu_info()
        HWInfo.sockets = get_number_of_sockets()
        HWInfo.cores = psutil.cpu_count(logical=False)
        HWInfo.cores_per_socket = int(psutil.cpu_count(logical=False) / self.sockets)
        HWInfo.threads_per_socket = int(psutil.cpu_count(logical=True) / self.sockets)
        HWInfo.total_memory = f"{psutil.virtual_memory().total / (1024 ** 3):.3f}GB"
        HWInfo.system = get_distribution()
        HWInfo.ip = determine_ip()
        if cpu_info.get("brand"):
            HWInfo.platform = cpu_info.get("brand")
        elif cpu_info.get("brand_raw"):
            HWInfo.platform = cpu_info.get("brand_raw")
        else:
            HWInfo.platform = ""
        HWInfo.hyperthreading_enabled = psutil.cpu_count(logical=False) != psutil.cpu_count(
            logical=True,
        )
        HWInfo.turboboost_enabled = is_turbo_boost_enabled()
        HWInfo.bios_version = get_bios_version()
        HWInfo.kernel = get_kernel_version()
        try:
            min_cpu_freq = int(psutil.cpu_freq(percpu=False).min)
            max_cpu_freq = int(psutil.cpu_freq(percpu=False).max)
        except Exception:
            log.warning("Cannot collect cpu frequency information.")
            min_cpu_freq = 0
            max_cpu_freq = 0
        finally:
            if min_cpu_freq == 0:
                HWInfo.min_cpu_freq = "n/a"
            else:
                HWInfo.min_cpu_freq = f"{min_cpu_freq}Hz"
            if max_cpu_freq == 0:
                HWInfo.max_cpu_freq = "n/a"
            else:
                HWInfo.max_cpu_freq = f"{max_cpu_freq}Hz"
        HWInfo.initialized = True


def get_number_of_sockets() -> int:
    """Get number of sockets in platform."""
    cmd = "lscpu | grep 'Socket(s)' | cut -d ':' -f 2"
    if psutil.WINDOWS:
        cmd = 'wmic cpu get DeviceID | find /c "CPU"'

    proc = subprocess.Popen(
        args=cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        universal_newlines=False,
    )
    proc.wait()
    if proc.stdout:
        for line in proc.stdout:
            return int(line.decode("utf-8", errors="ignore").strip())
    return 0


def get_distribution() -> str:
    """
    Return system distibution.

    :return: distribution name
    :rtype: str
    """
    return f"{platform.system()} {platform.release()}"


def get_bios_version() -> str:
    """Return bios version."""
    if psutil.LINUX:
        try:
            cmd = "cat /sys/class/dmi/id/bios_version"

            proc = subprocess.Popen(
                args=cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=False,
            )
            proc.wait()
            bios_version = "n/a"
            if proc.stdout and proc.returncode == 0:
                for line in proc.stdout:
                    bios_version = line.decode("utf-8", errors="ignore").strip()

            cmd = "grep microcode  /proc/cpuinfo -m1"
            proc = subprocess.Popen(
                args=cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=False,
            )
            proc.wait()
            microcode = "n/a"
            if proc.stdout and proc.returncode == 0:
                for line in proc.stdout:
                    microcode_raw = line.decode("utf-8", errors="ignore").strip()
                    result = re.search(r"microcode.*: (.*)", microcode_raw)
                    if result:
                        microcode = result.group(1)

            return f"BIOS {bios_version} Microcode {microcode}"
        except Exception:
            return "n/a"
    return "n/a"


def is_turbo_boost_enabled() -> Union[str, bool]:
    """
    Check if turbo boost is enabled.

    Return True if enabled, False if disabled, None if cannot collect info.
    """
    if psutil.LINUX:
        try:
            cmd = "cat /sys/devices/system/cpu/intel_pstate/no_turbo"
            proc = subprocess.Popen(
                args=cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=False,
            )
            proc.wait()
            if proc.stdout:
                for line in proc.stdout:
                    raw_status = int(line.decode("utf-8", errors="ignore").strip())
                    if raw_status == 0:
                        return True
                    elif raw_status == 1:
                        return False
                    else:
                        return "n/a"
        except Exception:
            return "n/a"
    return "n/a"


def get_kernel_version() -> str:
    """Return kernel version."""
    if psutil.LINUX:
        return platform.release()
    else:
        return platform.platform()



"""Tests for the dataloader module."""
import unittest
import os
import numpy as np
import shutil
from neural_compressor.utils.create_obj_from_config import create_dataset, create_dataloader
from neural_compressor.data.dataloaders.dataloader import DataLoader
from neural_compressor.data import DATASETS, DATALOADERS, TRANSFORMS
from PIL import Image

class TestBuiltinDataloader(unittest.TestCase):
    @classmethod
    def tearDownClass(cls):
        os.remove('./t10k-labels-idx1-ubyte.gz')
        os.remove('./t10k-images-idx3-ubyte.gz')
        os.remove('./train-images-idx3-ubyte.gz')
        os.remove('./train-labels-idx1-ubyte.gz')
        os.remove('./mnist.npz')

    def test_pytorch_dataset(self):
        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR10": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'pytorch', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR100": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'pytorch', dataloader_args)

        dataloader_args = {
            'dataset': {"MNIST": {'root': './test', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'pytorch', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"MNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('pytorch', dataloader_args)
        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24))
            break

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"FashionMNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('pytorch', dataloader_args)
        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24))
            break

        dataloader_args = {
            'batch_size': 2,
            'shuffle': True,
            'dataset': {"FashionMNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('pytorch', dataloader_args)
        self.assertEqual(dataloader.dataloader.sampler.__class__.__name__, 'RandomSampler')
        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24))
            break

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"FashionMNIST": {'root': './', 'train':True, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None,
            'distributed': True
        }
        dataloader = create_dataloader('pytorch', dataloader_args)
        self.assertEqual(dataloader.dataloader.sampler.__class__.__name__, 'DistributedSampler')
        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24))
            break

    def test_mxnet_dataset(self):
        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR10": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'mxnet', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR100": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'mxnet', dataloader_args)

        dataloader_args = {
            'dataset': {"MNIST": {'root': './test', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'mxnet', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"MNIST": {'root': './', 'train':True, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('mxnet', dataloader_args)

        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24,1))
            break

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"FashionMNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('mxnet', dataloader_args)

        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24,1))
            break

        dataloader_args = {
            'batch_size': 2,
            'shuffle': True,
            'dataset': {"MNIST": {'root': './', 'train':True, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        with self.assertLogs() as cm:
            dataloader = create_dataloader('mxnet', dataloader_args)
        self.assertEqual(cm.output, ['WARNING:root:Shuffle is not supported yet in' \
                         ' MXNetDataLoader, ignoring shuffle keyword.'])

    def test_tf_dataset(self):
        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR10": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'tensorflow', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR100": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'tensorflow', dataloader_args)

        dataloader_args = {
            'dataset': {"MNIST": {'root': './test', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader, 'tensorflow', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"MNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('tensorflow', dataloader_args)

        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24))
            break

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"FashionMNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('tensorflow', dataloader_args)

        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24))
            break

        dataloader_args = {
            'batch_size': 2,
            'shuffle': True,
            'dataset': {"FashionMNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        with self.assertLogs() as cm:
            dataloader = create_dataloader('tensorflow', dataloader_args)
        self.assertEqual(cm.output, ['WARNING:root:Shuffle is not supported yet in' \
                         ' TensorflowDataLoader, ignoring shuffle keyword.'])

    def test_onnx_dataset(self):
        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR10": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader,
                            'onnxrt_qlinearops', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"CIFAR100": {'root': './', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader,
                            'onnxrt_qlinearops', dataloader_args)

        dataloader_args = {
            'dataset': {"MNIST": {'root': './test', 'train':False, 'download':False}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        self.assertRaises(RuntimeError, create_dataloader,
                                'onnxrt_qlinearops', dataloader_args)

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"MNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('onnxrt_qlinearops', dataloader_args)

        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24,1))
            break

        dataloader_args = {
            'batch_size': 2,
            'dataset': {"FashionMNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('onnxrt_qlinearops', dataloader_args)

        for data in dataloader:
            self.assertEqual(len(data[0]), 2)
            self.assertEqual(data[0][0].shape, (24,24,1))
            break

        dataloader_args = {
            'batch_size': 2,
            'shuffle': True,
            'dataset': {"FashionMNIST": {'root': './', 'train':False, 'download':True}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        with self.assertLogs() as cm:
            dataloader = create_dataloader('onnxrt_qlinearops', dataloader_args)
        self.assertEqual(cm.output, ['WARNING:root:Shuffle is not supported yet' \
                         ' in ONNXRTDataLoader, ignoring shuffle keyword.'])

class TestImagenetRaw(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        os.makedirs('val', exist_ok=True)
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('val/test.jpg')
        with open('val/val.txt', 'w') as f:
            f.write('test.jpg   0')

    @classmethod
    def tearDownClass(cls):
        if os.path.exists('val'):
            shutil.rmtree('val')

    def test_tensorflow(self):
        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path': './val', 'image_list':None}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('tensorflow', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path':'val', 'image_list':'val/val.txt'}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('tensorflow', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

    def test_pytorch(self):
        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path': 'val', 'image_list':None}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('pytorch', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path':'val', 'image_list':'val/val.txt'}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('pytorch', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

    def test_mxnet(self):
        import mxnet as mx
        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path': 'val', 'image_list':None}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('mxnet', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path':'val', 'image_list':'val/val.txt'}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('mxnet', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

    def test_onnx(self):
        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path': 'val', 'image_list':None}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('onnxrt_integerops', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path':'val', 'image_list':'val/val.txt'}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('onnxrt_integerops', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break
        # test old api
        eval_dataset = create_dataset('onnxrt_integerops', {'Imagenet':{'root':'./'}}, None, None)
        dataloader = DataLoader('onnxrt_integerops', dataset=eval_dataset, batch_size=1)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (100,100,3))
            break

        with open('val/fake_map.txt', 'w') as f:
            f.write('test.jpg   0 \n')
            f.write('test2.jpg   1')
        dataset_args = {
            "ImagenetRaw": {'data_path':'val', 'image_list':'val/fake_map.txt'},
        }
        dataset = create_dataset('onnxrt_integerops', dataset_args, None, None)
        self.assertEqual(len(dataset), 1)

        with open('val/fake_map.txt', 'w') as f:
            f.write('test2.jpg   1')
        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path':'val', 'image_list':'val/fake_map.txt'}},
            'transform': None,
            'filter': None
        }
        self.assertRaises(ValueError, create_dataloader, 'onnxrt_integerops', dataloader_args)

        with open('val/not_found_map.txt', 'w') as f:
            f.write('test.jpg   0' + '\n')
            f.write('not_found.jpg   1')
        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path':'val', 'image_list':'val/not_found_map.txt'}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('onnxrt_integerops', dataloader_args)
        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

        with open('val/blank.txt', 'w') as f:
            f.write('blank.jpg   0')
        dataloader_args = {
            'dataset': {"ImagenetRaw": {'data_path':'val', 'image_list':'val/blank.txt'}},
            'transform': None,
            'filter': None
        }
        self.assertRaises(ValueError, create_dataloader,
                            'onnxrt_qlinearops', dataloader_args)


class TestImageFolder(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        os.makedirs('val', exist_ok=True)
        os.makedirs('val/0', exist_ok=True)
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('val/0/test.jpg')

    @classmethod
    def tearDownClass(cls):
        if os.path.exists('val'):
            shutil.rmtree('val')

    def test_tensorflow(self):
        dataloader_args = {
            'dataset': {"ImageFolder": {'root': './val'}},
            'transform': {'RandomResizedCrop': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('tensorflow', dataloader_args)

        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

    def test_pytorch(self):
        dataloader_args = {
            'dataset': {"ImageFolder": {'root': './val'}},
            'transform': {'Resize': {'size': 24}, 'ToTensor':{}},
            'filter': None
        }
        dataloader = create_dataloader('pytorch', dataloader_args)

        for data in dataloader:
            self.assertEqual(data[0][0].shape, (3,24,24))
            break

    def test_mxnet(self):
        dataloader_args = {
            'dataset': {"ImageFolder": {'root': './val'}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('mxnet', dataloader_args)

        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

    def test_onnx(self):
        dataloader_args = {
            'dataset': {"ImageFolder": {'root': './val'}},
            'transform': {'Resize': {'size': 24}},
            'filter': None
        }
        dataloader = create_dataloader('onnxrt_integerops', dataloader_args)

        for data in dataloader:
            self.assertEqual(data[0][0].shape, (24,24,3))
            break

class TestDataloader(unittest.TestCase):
    def test_iterable_dataset(self):
        class iter_dataset(object):
            def __iter__(self):
                for i in range(100):
                    yield np.zeros([256, 256, 3])
        dataset = iter_dataset()
        data_loader = DATALOADERS['tensorflow'](dataset)
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (1, 256, 256, 3))

    def test_tensorflow_bert(self):
        import collections
        import tensorflow as tf
        import json
        label = [{
            "paragraphs":[
                {'context':
                    'Super Bowl 50 was an American football game to determine the champion of the National Football League (NFL) for the 2015 season.',
                'qas': [{
                    'answers': [
                        {'answer_start': 177, 'text': 'Denver Broncos'},
                        {'answer_start': 177, 'text': 'Denver Broncos'},
                        {'answer_start': 177, 'text': 'Denver Broncos'}],
                    'question': 'Which NFL team represented the AFC at Super Bowl 50?',
                    'id': '56be4db0acb8001400a502ec'}]
                }
            ]
        }]
        fake_json = json.dumps({'data': label, 'version': '1.1'})
        with open('dev.json', 'w') as f:
            f.write(fake_json)
        unique_id = 1000000000
        example_index = 0
        doc_span_index = 0
        tokens = ['[SEP]', 'super', 'bowl', '50', 'was', 'an', 'american', 'football', 'game', 'to', 'determine', \
            'the', 'champion', 'of', 'the', 'national', 'football', 'league', '(', 'nfl', ')', 'for', 'the', \
            '2015', 'season', '.', '[SEP]']
        token_to_orig_map = {13: 0, 14: 1, 15: 2, 16: 3, 17: 4, 18: 5, 19: 6, 20: 7, 21: 8, 22: 9, 23: 10, 24: 11, 25: 12, \
            26: 13, 27: 14, 28: 15, 29: 16, 30: 17, 31: 17, 32: 17, 33: 18, 34: 19, 35: 20, 36: 21, 37: 21, \
            38: 22, 39: 23, 40: 24, 41: 25}
        token_is_max_context = {13: True, 14: True, 15: True, 16: True, 17: True, 18: True, 19: True, 20: True, 21: True, 22: True, \
            23: True, 24: True, 25: True, 26: True, 27: True, 28: True, 29: True, 30: True, 31: True, 32: True, 33: True, 34: True, \
            35: True, 36: True, 37: True, 38: True, 39: True, 40: True, 41: True}
        input_ids = [101, 2029, 5088, 2136, 3421, 1996, 10511, 2012, 3565, 4605, 2753, 1029, 102, 3565, 4605, 2753,\
            1007, 2005, 1996, 2325, 2161, 1012, 1996, 2137, 2374, 3034, 1006]
        input_mask = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
        segment_ids = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
        with tf.io.TFRecordWriter('./test.record') as writer:
            features = collections.OrderedDict()
            features["unique_ids"] = tf.train.Feature(
                int64_list=tf.train.Int64List(value=list([unique_id])))
            features["input_ids"] = tf.train.Feature(
                int64_list=tf.train.Int64List(value=list(input_ids)))
            features["input_mask"] = tf.train.Feature(
                int64_list=tf.train.Int64List(value=list(input_mask)))
            features["segment_ids"] = tf.train.Feature(
                int64_list=tf.train.Int64List(value=list(segment_ids)))
            tf_example = tf.train.Example(features=tf.train.Features(feature=features))
            writer.write(tf_example.SerializeToString())
        eval_dataset = create_dataset(
            'tensorflow',
            {'bert':{'root':'test.record', 'label_file': './dev.json'}},
            None,
            None)
        dataloader = DATALOADERS['tensorflow'](dataset=eval_dataset, batch_size=1)
        for inputs, labels in dataloader:
            self.assertEqual(inputs[1], 1)
            self.assertEqual(len(labels), 1)
        os.remove('test.record')
        os.remove('dev.json')

    def test_onnx_imagenet(self):
        os.makedirs('val', exist_ok=True)
        os.makedirs('val/0', exist_ok=True)
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('val/test.jpg')
        args = {'ImageFolder': {'root': './val'}}
        ds = create_dataset('onnxrt_qlinearops', args, None, None)
        dataloader = DATALOADERS['onnxrt_qlinearops'](ds)
        for image, label in dataloader:
            self.assertEqual(image[0].size, (100,100))
        shutil.rmtree('val')

    def test_voc_record(self):
        import six
        import collections
        import tensorflow as tf
        tf.compat.v1.disable_eager_execution()

        def _bytes_list_feature(values):
            def norm2bytes(value):
                return value.encode() if isinstance(value, str) and six.PY3 else value
            return tf.train.Feature(
                bytes_list=tf.train.BytesList(value=[norm2bytes(values)]))

        def _int64_list_feature(values):
            if not isinstance(values, collections.Iterable):
                values = [values]
            return tf.train.Feature(int64_list=tf.train.Int64List(value=values))

        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test.jpg')
        random_array = np.random.random_sample([100,100,3]) * 0
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test.png')
        image_data = tf.compat.v1.gfile.GFile('test.jpg', 'rb').read()
        seg_data = tf.compat.v1.gfile.GFile('test.png', 'rb').read()
        filename = 'test'

        example = tf.train.Example(features=tf.train.Features(feature={
            'image/encoded': _bytes_list_feature(image_data),
            'image/filename': _bytes_list_feature(filename),
            'image/format': _bytes_list_feature('png'),
            'image/height': _int64_list_feature(100),
            'image/width': _int64_list_feature(100),
            'image/channels': _int64_list_feature(3),
            'image/segmentation/class/encoded': (
                _bytes_list_feature(seg_data)),
            'image/segmentation/class/format': _bytes_list_feature('png'),
        }))

        if not os.path.exists('./test_record'):
            os.mkdir('./test_record')
        with tf.io.TFRecordWriter('./test_record/val-test.record') as writer:
            writer.write(example.SerializeToString())
        eval_dataset = create_dataset(
            'tensorflow', {'VOCRecord':{'root':'./test_record'}}, {'ParseDecodeVoc':{}}, None)
        dataloader = DATALOADERS['tensorflow'](dataset=eval_dataset, batch_size=1)
        for (inputs, labels) in dataloader:
            self.assertEqual(inputs.shape, (1,100,100,3))
            self.assertEqual(labels[0].shape, (100,100,1))

        os.remove('./test_record/val-test.record')
        os.remove('test.jpg')
        os.remove('test.png')
        shutil.rmtree('./test_record')

    def test_coco_record(self):
        import tensorflow as tf
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test.jpeg')

        image = tf.compat.v1.gfile.FastGFile('test.jpeg','rb').read()
        source_id = '000000397133.jpg'.encode('utf-8')
        label = 'person'.encode('utf-8')
        example = tf.train.Example(features=tf.train.Features(feature={
            'image/encoded':tf.train.Feature(
                bytes_list=tf.train.BytesList(value=[image])),
            'image/object/class/text':tf.train.Feature(
                bytes_list=tf.train.BytesList(value=[label])),
            'image/source_id':tf.train.Feature(
                bytes_list=tf.train.BytesList(value=[source_id])),
            'image/object/bbox/xmin':tf.train.Feature(
                float_list=tf.train.FloatList(value=[10])),
            'image/object/bbox/ymin':tf.train.Feature(
                float_list=tf.train.FloatList(value=[10])),
            'image/object/bbox/xmax':tf.train.Feature(
                float_list=tf.train.FloatList(value=[100])),
            'image/object/bbox/ymax':tf.train.Feature(
                float_list=tf.train.FloatList(value=[100])),
        }))

        with tf.io.TFRecordWriter('test.record') as writer:
            writer.write(example.SerializeToString())
        eval_dataset = create_dataset(
            'tensorflow',
            {'COCORecord':{'root':'test.record'}},
            {'RandomVerticalFlip': {},
            'RandomHorizontalFlip': {},
            'CropResize':{'x':0, 'y':0, 'width':10, 'height':10, 'size':[5,5]},
            'Transpose':{'perm': [2, 0, 1]}
            },
            None)
        dataloader = DATALOADERS['tensorflow'](dataset=eval_dataset, batch_size=1)
        for inputs, labels in dataloader:
            self.assertEqual(inputs.shape, (1,3,5,5))
            self.assertEqual(labels[0].shape, (1,1,4))
        os.remove('test.record')
        os.remove('test.jpeg')

    def test_coco_record_disable_eager(self):
        import tensorflow as tf
        tf.compat.v1.disable_eager_execution()
        self.test_coco_record()

    def test_coco_raw(self):
        import json
        import collections
        from neural_compressor.data import TRANSFORMS
        import mxnet as mx
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test_0.jpg')
        im.save('test_1.jpg')
        fake_dict = {
            'info': {
                'description': 'COCO 2017 Dataset',
                'url': 'http://cocodataset.org',
                'version': '1.0',
                'year': 2017,
                'contributor': 'COCO Consortium',
                'date_created': '2017/09/01'
            },
            'licenses':{

            },
            'images':[{
                'file_name': 'test_0.jpg',
                'height': 100,
                'width': 100,
                'id': 0
            },
            {
                'file_name': 'test_1.jpg',
                'height': 100,
                'width': 100,
                'id': 1
            },
            {
                'file_name': 'test_2.jpg',
                'height': 100,
                'width': 100,
                'id': 2
            }],
            'annotations':[{
                'category_id': 18,
                'id': 1767,
                'iscrowd': 0,
                'image_id': 0,
                'bbox': [473.07, 395.93, 38.65, 28.67],
            },
            {
                'category_id': 18,
                'id': 1768,
                'iscrowd': 0,
                'image_id': 1,
                'bbox': [473.07, 395.93, 38.65, 28.67],
            },
            {
                'category_id': 18,
                'id': 1769,
                'iscrowd': 0,
                'image_id': 2,
                'bbox': [],
            }],
            'categories':[{
                'supercategory': 'animal',
                'id': 18,
                'name': 'dog'
            }]
        }
        fake_json = json.dumps(fake_dict)
        with open('anno.json', 'w') as f:
            f.write(fake_json)

        args = {'COCORaw': {'root': './', 'img_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('tensorflow', args, None, None)
        dataloader = DATALOADERS['tensorflow'](ds)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        trans_args = {'Transpose': {'perm': [2, 0, 1]}}
        ds = create_dataset('tensorflow', args, trans_args, None)
        dataloader = DATALOADERS['tensorflow'](ds)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (3,100,100))

        args = {'COCORaw': {'root': './', 'img_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('onnxrt_qlinearops', args, None, None)
        dataloader = DATALOADERS['onnxrt_qlinearops'](ds)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        args = {'COCORaw': {'root': './', 'img_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('mxnet', args, None, None)
        def collate(batch):
            elem = batch[0]
            if isinstance(elem, mx.ndarray.NDArray):
                return mx.nd.stack(*batch)
            elif isinstance(elem, collections.abc.Sequence):
                batch = zip(*batch)
                return [collate(samples) for samples in batch]
            elif isinstance(elem, collections.abc.Mapping):
                return {key: collate([d[key] for d in batch]) for key in elem}
            elif isinstance(elem, np.ndarray):
                return np.stack(batch)
            else:
                return batch
        dataloader = DATALOADERS['mxnet'](ds, collate_fn=collate)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        args = {'COCORaw': {'root': './', 'img_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('pytorch', args, None, None)
        def collate(batch):
            elem = batch[0]
            if isinstance(elem, collections.abc.Mapping):
                return {key: collate([d[key] for d in batch]) for key in elem}
            elif isinstance(elem, collections.abc.Sequence):
                batch = zip(*batch)
                return [collate(samples) for samples in batch]
            elif isinstance(elem, np.ndarray):
                return np.stack(batch)
            else:
                return batch
        dataloader = DATALOADERS['pytorch'](dataset=ds, collate_fn=collate)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        os.remove('test_0.jpg')
        os.remove('test_1.jpg')
        os.remove('anno.json')

    def test_coco_npy(self):
        import json
        import collections
        from neural_compressor.data import TRANSFORMS
        import mxnet as mx
        import cv2
        import numpy as np

        def maybe_resize(img, dims):
            img = np.array(img, dtype=np.float32)
            if len(img.shape) < 3 or img.shape[2] != 3:
                # some images might be grayscale
                img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGB)
            img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
            if dims != None:
                im_height, im_width, _ = dims
                img = cv2.resize(img, (im_width, im_height), interpolation=cv2.INTER_LINEAR)
            return img

        def convert_npy(src):
            img = cv2.imread(src)
            dims = [100, 100, 3]
            image_format="NHWC"
            need_transpose = True if image_format == "NCHW" else False
            img = maybe_resize(img, dims)
            mean = np.array([0.485, 0.456, 0.406], dtype=np.float32)
            std = np.array([0.229, 0.224, 0.225], dtype=np.float32)
            img = img / 255. - mean
            img = img / std
            if need_transpose:
                img = img.transpose([2, 0, 1])
            return img

        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test_0.jpg')
        im.save('test_1.jpg')
        im_npy = convert_npy('test_0.jpg')
        np.save('test_0.jpg.npy', im_npy)
        np.save('test_1.jpg.npy', im_npy)

        fake_dict = {
            'info': {
                'description': 'COCO 2017 Dataset',
                'url': 'http://cocodataset.org',
                'version': '1.0',
                'year': 2017,
                'contributor': 'COCO Consortium',
                'date_created': '2017/09/01'
            },
            'licenses':{

            },
            'images':[{
                'file_name': 'test_0.jpg',
                'height': 100,
                'width': 100,
                'id': 0
            },
            {
                'file_name': 'test_1.jpg',
                'height': 100,
                'width': 100,
                'id': 1
            },
            {
                'file_name': 'test_2.jpg',
                'height': 100,
                'width': 100,
                'id': 2
            }],
            'annotations':[{
                'category_id': 18,
                'id': 1767,
                'iscrowd': 0,
                'image_id': 0,
                'bbox': [473.07, 395.93, 38.65, 28.67],
            },
            {
                'category_id': 18,
                'id': 1768,
                'iscrowd': 0,
                'image_id': 1,
                'bbox': [473.07, 395.93, 38.65, 28.67],
            },
            {
                'category_id': 18,
                'id': 1769,
                'iscrowd': 0,
                'image_id': 2,
                'bbox': [],
            }],
            'categories':[{
                'supercategory': 'animal',
                'id': 18,
                'name': 'dog'
            }]
        }
        fake_json = json.dumps(fake_dict)
        with open('anno.json', 'w') as f:
            f.write(fake_json)

        args = {'COCONpy': {'root': './', 'npy_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('tensorflow', args, None, None)
        dataloader = DATALOADERS['tensorflow'](ds)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        args = {'COCONpy': {'root': './', 'npy_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('onnxrt_qlinearops', args, None, None)
        dataloader = DATALOADERS['onnxrt_qlinearops'](ds)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        args = {'COCONpy': {'root': './', 'npy_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('mxnet', args, None, None)
        def collate(batch):
            elem = batch[0]
            if isinstance(elem, mx.ndarray.NDArray):
                return mx.nd.stack(*batch)
            elif isinstance(elem, collections.abc.Sequence):
                batch = zip(*batch)
                return [collate(samples) for samples in batch]
            elif isinstance(elem, collections.abc.Mapping):
                return {key: collate([d[key] for d in batch]) for key in elem}
            elif isinstance(elem, np.ndarray):
                return np.stack(batch)
            else:
                return batch
        dataloader = DATALOADERS['mxnet'](ds, collate_fn=collate)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        args = {'COCONpy': {'root': './', 'npy_dir': '', 'anno_dir': 'anno.json'}}
        ds = create_dataset('pytorch', args, None, None)
        def collate(batch):
            elem = batch[0]
            if isinstance(elem, collections.abc.Mapping):
                return {key: collate([d[key] for d in batch]) for key in elem}
            elif isinstance(elem, collections.abc.Sequence):
                batch = zip(*batch)
                return [collate(samples) for samples in batch]
            elif isinstance(elem, np.ndarray):
                return np.stack(batch)
            else:
                return batch
        dataloader = DATALOADERS['pytorch'](dataset=ds, collate_fn=collate)
        for image, label in dataloader:
            self.assertEqual(image[0].shape, (100,100,3))

        os.remove('test_0.jpg')
        os.remove('test_1.jpg')
        os.remove('test_0.jpg.npy')
        os.remove('test_1.jpg.npy')
        os.remove('anno.json')

    def test_tensorflow_imagenet_dataset(self):
        import tensorflow as tf
        tf.compat.v1.disable_eager_execution()
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test.jpeg')

        image = tf.compat.v1.gfile.FastGFile('test.jpeg','rb').read()
        example = tf.train.Example(features=tf.train.Features(feature={
            'image/encoded':tf.train.Feature(
                    bytes_list=tf.train.BytesList(value=[image])),
            'image/class/label':tf.train.Feature(
                    int64_list=tf.train.Int64List(value=[1])),
        }))

        with tf.io.TFRecordWriter('validation-00000-of-00000') as writer:
            writer.write(example.SerializeToString())

        eval_dataset = create_dataset(
            'tensorflow', {'ImageRecord':{'root':'./'}}, None, None)

        dataloader = DATALOADERS['tensorflow'](dataset=eval_dataset, batch_size=1)
        for (inputs, labels) in dataloader:
            self.assertEqual(inputs.shape, (1,100,100,3))
            self.assertEqual(labels.shape, (1, 1))

        # test old api
        eval_dataset = create_dataset(
            'tensorflow', {'Imagenet':{'root':'./'}}, None, None)
        dataloader = DataLoader('tensorflow', dataset=eval_dataset, batch_size=1)
        for (inputs, labels) in dataloader:
            self.assertEqual(inputs.shape, (1,100,100,3))
            self.assertEqual(labels.shape, (1, 1))

        os.remove('validation-00000-of-00000')
        os.remove('test.jpeg')

    def test_pytorch_bert_dataset(self):
        dataset = [[
           [101,2043,2001],
           [1,1,1],
           [[0,0,0,0,0,0,0],
            [0,0,0,0,0,0,0],
            [0,0,0,0,0,0,0]],
           [1,1,1],
           [1,1,1],
           [[0,0,0,0,0,0,0],
            [0,0,0,0,0,0,0],
            [0,0,0,0,0,0,0]]
        ]]
        with self.assertRaises(AssertionError):
            create_dataset('pytorch', {'bert': {'dataset':dataset, 'task':'test'}},
                            None, None)

        ds = create_dataset(
            'pytorch',
            {'bert': {'dataset':dataset, 'task':'classifier', 'model_type':'distilbert'}},
            None, None)
        self.assertEqual(len(ds), 1)
        self.assertEqual(3, len(ds[0][0]))

        ds = create_dataset(
            'pytorch',
            {'bert': {'dataset':dataset, 'task':'classifier', 'model_type':'bert'}},
            None, None)
        self.assertEqual(4, len(ds[0][0]))

        ds = create_dataset(
            'pytorch', {'bert': {'dataset':dataset, 'task':'squad'}}, None, None)
        self.assertEqual(3, len(ds[0][0]))

        ds = create_dataset(
            'pytorch',
            {'bert': {'dataset':dataset, 'task':'squad', 'model_type':'distilbert'}},
            None, None)
        self.assertEqual(2, len(ds[0][0]))

        ds = create_dataset(
            'pytorch',
            {'bert': {'dataset':dataset, 'task':'squad', 'model_type':'xlnet'}},
            None, None)
        self.assertEqual(5, len(ds[0][0]))

    def test_tensorflow_dummy(self):
        datasets = DATASETS('tensorflow')
        dataset = datasets['dummy'](shape=(4, 256, 256, 3))

        data_loader = DATALOADERS['tensorflow'](dataset)
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (1, 256, 256, 3))
        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (2, 256, 256, 3))

        with self.assertRaises(AssertionError):
            dataset = datasets['dummy'](shape=[(4, 256, 256, 3), (256, 256, 3)])
        with self.assertRaises(AssertionError):
            dataset = datasets['dummy'](shape=(4, 256, 256, 3), low=[1., 0.])
        with self.assertRaises(AssertionError):
            dataset = datasets['dummy'](shape=(4, 256, 256, 3), high=[128., 127.])
        with self.assertRaises(AssertionError):
            dataset = datasets['dummy'](shape=(4, 256, 256, 3), dtype=['float32', 'int8'])

    def test_tensorflow_dummy_v2(self):
        datasets = DATASETS('tensorflow')
        # test with label
        dataset = datasets['dummy_v2'](\
            input_shape=(256, 256, 3), label_shape=(1,))
        data_loader = DATALOADERS['tensorflow'](dataset)
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data[0].shape, (1, 256, 256, 3))
        self.assertEqual(data[1].shape, (1, 1))
        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data[0].shape, (2, 256, 256, 3))
        self.assertEqual(data[1].shape, (2, 1))

        # test without label
        dataset = datasets['dummy_v2'](input_shape=(256, 256, 3))
        data_loader = DATALOADERS['tensorflow'](dataset)
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (1, 256, 256, 3))
        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (2, 256, 256, 3))

        with self.assertRaises(AssertionError):
            dataset = datasets['dummy_v2'](\
                input_shape=(256, 256, 3), low=[1., 0.])
        with self.assertRaises(AssertionError):
            dataset = datasets['dummy_v2'](\
                input_shape=(256, 256, 3), high=[128., 127.])
        with self.assertRaises(AssertionError):
            dataset = datasets['dummy_v2'](\
                input_shape=(256, 256, 3), dtype=['float32', 'int8'])

    def test_style_transfer_dataset(self):
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test.jpg')

        datasets = DATASETS('tensorflow')
        dataset = datasets['style_transfer'](content_folder='./', style_folder='./')
        length = len(dataset)
        image, label = dataset[0]
        self.assertEqual(image[0].shape, (256, 256, 3))
        self.assertEqual(image[1].shape, (256, 256, 3))
        os.remove('test.jpg')

    def test_tensorflow_list_dict(self):
        dataset = [{'a':1, 'b':2, 'c':3, 'd':4}, {'a':5, 'b':6, 'c':7, 'd':8}]
        data_loader = DATALOADERS['tensorflow'](dataset)

        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data, {'a':[1], 'b':[2], 'c':[3], 'd':[4]})

        # test iterable consistent
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data, {'a':[1], 'b':[2], 'c':[3], 'd':[4]})

        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data, {'a':[1, 5], 'b':[2, 6], 'c':[3, 7], 'd':[4, 8]})

    # def test_tensorflow2_dataset(self):
    #     dataset = [[1, 2, 3, 4], [5, 6, 7, 8]]
    #     dataset = np.array(dataset)
    #     import tensorflow as tf
    #     dataset = tf.data.Dataset.from_tensors(dataset)
    #     data_loader = DATALOADERS['tensorflow'](dataset)

    #     iterator = iter(data_loader)
    #     data = next(iterator)
    #     self.assertEqual(data[0][1], 2)

    def test_pytorch_dummy(self):
        datasets = DATASETS('pytorch')
        transform = TRANSFORMS('pytorch', 'preprocess')['Resize'](**{'size':100})
        dataset = datasets['dummy'](shape=[(4, 256, 256, 3), (4, 1)], \
            high=[10., 10.], low=[0., 0.], transform=transform)

        data_loader = DATALOADERS['pytorch'](dataset)
        iterator = iter(data_loader)
        data, label = next(iterator)
        self.assertEqual(data.shape, (1, 256, 256, 3))
        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data, label = next(iterator)
        self.assertEqual(data.shape, (2, 256, 256, 3))

    def test_mxnet_dummy(self):
        datasets = DATASETS('mxnet')
        transform = TRANSFORMS('mxnet', 'preprocess')['Resize'](**{'size':100})
        dataset = datasets['dummy'](shape=(4, 256, 256, 3), transform=transform)

        data_loader = DATALOADERS['mxnet'](dataset)
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (1, 256, 256, 3))
        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (2, 256, 256, 3))

        dataset = datasets['dummy'](shape=(4, 256, 256, 3), label=True)
        self.assertEqual(dataset[0][1], 0)

    def test_onnxrt_qlinear_dummy(self):
        datasets = DATASETS('onnxrt_qlinearops')
        transform = TRANSFORMS('onnxrt_qlinearops', 'preprocess')['Resize'](**{'size':100})
        dataset = datasets['dummy'](shape=(4, 256, 256, 3), transform=transform)

        data_loader = DATALOADERS['onnxrt_qlinearops'](dataset)
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (1, 256, 256, 3))
        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (2, 256, 256, 3))

    def test_onnx_integer_dummy(self):
        datasets = DATASETS('onnxrt_integerops')
        dataset = datasets['dummy'](shape=(4, 256, 256, 3))

        data_loader = DATALOADERS['onnxrt_integerops'](dataset)
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (1, 256, 256, 3))
        # dynamic batching
        data_loader.batch(batch_size=2, last_batch='rollover')
        iterator = iter(data_loader)
        data = next(iterator)
        self.assertEqual(data.shape, (2, 256, 256, 3))

    def test_onnx_bert(self):
        import csv
        os.mkdir('./MRPC')
        with open('./MRPC/msr_paraphrase_test.txt', 'a') as f:
            f.write('Quality #1 ID   #2 ID   #1 String   #2 String\n')
            f.write("1   1089874 1089925 PCCW 's chief operating officer , Mike Butcher , and Alex Arena , the chief financial officer , will report directly to Mr So . Current Chief Operating Officer Mike Butcher and Group Chief Financial Officer Alex Arena will report to So .")
        with open('./MRPC/msr_paraphrase_train.txt', 'a') as f:
            f.write('Quality #1 ID   #2 ID   #1 String   #2 String\n')
            f.write("""1   702876  702977  Amrozi accused his brother , whom he called " the witness " , of deliberately distorting his evidence . Referring to him as only " the witness " , Amrozi accused his brother of deliberately distorting his evidence .""")
        with open('./MRPC/dev.tsv', 'a') as f:
            tsv_w = csv.writer(f, delimiter='\t')
            tsv_w.writerow(['Quality', '#1 ID', '#2 ID', '#1 String', '#2 String'])
            tsv_w.writerow(['1', '1355540', '1355592', "He said the foodservice pie business doesn 'tfit thecompany 's long-term growth strategy .", "The foodservice pie businessdoes notfit our long-term growth strategy ."])
        with open('./MRPC/dev_ids.tsv', 'a') as f:
            tsv_w = csv.writer(f, delimiter='\t')
            tsv_w.writerow(['1606495', '1606619'])
        with open('./MRPC/test.tsv', 'a') as f:
            tsv_w = csv.writer(f, delimiter='\t')
            tsv_w.writerow(['index', '#1 ID', '#2 ID', '#1 String', '#2 String'])
            tsv_w.writerow(['0', '1089874', '1089925', "PCCW 's chief operating officer , Mike Butcher , and Alex Arena , the chief financial officer , will report directly to Mr So .", "Current Chief Operating Officer Mike Butcher and Group Chief Financial Officer Alex Arena will report to So ."])
        with open('./MRPC/train.tsv', 'a') as f:
            tsv_w = csv.writer(f, delimiter='\t')
            tsv_w.writerow(['Quality', '#1 ID', '#2 ID', '#1 String', '#2 String'])
            tsv_w.writerow(['1', '702876', '702977', """Amrozi accused his brother , whom he called " the witness " , of deliberately distorting his evidence .""", """Referring to him as only " the witness " , Amrozi accused his brother of deliberately distorting his evidence ."""])

        datasets = DATASETS('onnxrt_integerops')
        args = {'GLUE':
                    {'data_dir': './MRPC',
                     'model_name_or_path': 'bert-base-uncased',
                     'dynamic_length': True
                     }}
        ds = create_dataset('onnxrt_qlinearops', args, None, None)

        ds = create_dataset('onnxrt_qlinearops', args, None, None)
        dataloader = DATALOADERS['onnxrt_qlinearops'](ds)
        for inputs, label in dataloader:
            self.assertEqual(len(inputs), 3)
            self.assertEqual(inputs[0].shape[1], 48)
            self.assertEqual(len(label), 1)
            break
        shutil.rmtree('./dataset_cached')

        args = {'GLUE':
                    {'data_dir': './MRPC',
                     'model_type': 'roberta',
                     'model_name_or_path': 'roberta-base',
                     'dynamic_length': False
                     }}
        ds = create_dataset('onnxrt_qlinearops', args, None, None)
        dataloader = DATALOADERS['onnxrt_qlinearops'](ds)
        for inputs, label in dataloader:
            self.assertEqual(len(inputs), 2)
            self.assertEqual(inputs[0].shape[1], 128)
            self.assertEqual(len(label), 1)
            break

        shutil.rmtree('./MRPC')
        shutil.rmtree('./dataset_cached')


if __name__ == "__main__":
    unittest.main()
